@@ -84,18 +84,18 @@ class eseries():
                 self.dims = q.dims
                 self.shape = q.shape
             elif isinstance(q, (np.ndarray, list)):
-                q = np.asarray(q, dtype=object)
-                ind = np.shape(q)
-                num = ind[0]  # number of elements in q
+                num = len(q)  # number of elements in q
                 if any([Qobj(x).shape != Qobj(q[0]).shape for x in q]):
                     raise TypeError('All amplitudes must have same dimension.')
-                self.ampl = np.array([x for x in q], dtype=object)
-                self.rates = np.zeros(ind)
+                self.ampl = np.empty((num,), dtype=object)
+                self.ampl[:] = q
+                self.rates = np.zeros((num,))
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
             elif isinstance(q, Qobj):
                 qo = Qobj(q)
-                self.ampl = np.array([qo], dtype=object)
+                self.ampl = np.empty((1,), dtype=object)
+                self.ampl[0] = qo
                 self.rates = np.array([0])
                 self.dims = qo.dims
                 self.shape = qo.shape
@@ -107,18 +107,17 @@ class eseries():
 
         elif len(s) != 0:
             if isinstance(q, (np.ndarray, list)):
-                q = np.asarray(q, dtype=object)
-                ind = np.shape(q)
-                num = ind[0]
+                num = len(q)
                 if any([Qobj(x).shape != Qobj(q[0]).shape for x in q]):
                     raise TypeError('All amplitudes must have same dimension.')
-                self.ampl = np.array([Qobj(q[x]) for x in range(0, num)],
-                                     dtype=object)
+                self.ampl = np.empty((num,), dtype=object)
+                self.ampl[:] = [Qobj(qq) for qq in q]
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
             else:
                 num = 1
-                self.ampl = np.array([Qobj(q)], dtype=object)
+                self.ampl = np.empty((num,), dtype=object)
+                self.ampl[0] = Qobj(q)
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
 
@@ -134,13 +133,9 @@ class eseries():
                 self.rates = np.array(s)
 
         if len(self.ampl) != 0:
-            # combine arrays so that they can be sorted together
-            zipped = list(zip(self.rates, self.ampl))
-            zipped.sort()  # sort rates from lowest to highest
-            rates, ampl = list(zip(*zipped))  # get back rates and ampl
-            self.ampl = np.array(ampl, dtype=object)
-            self.rates = np.array(rates)
-
+            # Sort rates from lowest to highest.
+            order = np.argsort(self.rates)
+            self.ampl, self.rates = self.ampl[order], self.rates[order]
 
     def __str__(self):  # string of ESERIES information
         self.tidyup()
@@ -255,22 +250,17 @@ class eseries():
 
         if isinstance(self.ampl[0], Qobj):
             # amplitude vector contains quantum objects
-            val_list = []
+            val_list = np.empty((len(tlist),), dtype=object)
 
             for j in range(len(tlist)):
                 exp_factors = np.exp(np.array(self.rates) * tlist[j])
-
                 val = 0
                 for i in range(len(self.ampl)):
                     val += self.ampl[i] * exp_factors[i]
-
-                val_list.append(val)
-
-            val_list = np.array(val_list, dtype=object)
+                val_list[j] = val
         else:
             # the amplitude vector contains c numbers
             val_list = np.zeros(np.size(tlist), dtype=complex)
-
             for j in range(len(tlist)):
                 exp_factors = np.exp(np.array(self.rates) * tlist[j])
                 val_list[j] = np.sum(np.dot(self.ampl, exp_factors))
@@ -320,14 +310,12 @@ class eseries():
         ur_len = 0
 
         for r_idx in range(len(self.rates)):
-
             # look for a matching rate in the list of unique rates
             idx = -1
             for ur_key in unique_rates.keys():
                 if abs(self.rates[r_idx] - unique_rates[ur_key]) < rate_tol:
                     idx = ur_key
                     break
-
             if idx == -1:
                 # no matching rate, add it
                 unique_rates[ur_len] = self.rates[r_idx]
@@ -339,23 +327,21 @@ class eseries():
 
         # create new amplitude and rate list with only unique rates, and
         # nonzero amplitudes
-        self.rates = np.array([])
-        self.ampl = np.array([])
+        rates, ampl = [], []
         for ur_key in unique_rates.keys():
-            total_ampl = np.sum(np.asarray(ampl_dict[ur_key], dtype=object))
-
+            total_ampl = sum(ampl_dict[ur_key])
             if (isinstance(total_ampl, float) or
                     isinstance(total_ampl, complex)):
                 if abs(total_ampl) > ampl_tol:
-                    self.rates = np.append(self.rates, unique_rates[ur_key])
-                    self.ampl = np.append(self.ampl, total_ampl)
+                    rates.append(unique_rates[ur_key])
+                    ampl.append(total_ampl)
             else:
                 if abs(total_ampl.full()).max() > ampl_tol:
-                    self.rates = np.append(self.rates, unique_rates[ur_key])
-                    self.ampl = np.append(self.ampl,
-                                          np.asarray([total_ampl],
-                                                     dtype=object))
-
+                    rates.append(unique_rates[ur_key])
+                    ampl.append(total_ampl)
+        self.rates = np.array(rates)
+        self.ampl = np.empty((len(ampl),), dtype=object)
+        self.ampl[:] = ampl
         return self
 
 
