@@ -43,7 +43,7 @@ __all__ = ['floquet_modes', 'floquet_modes_t', 'floquet_modes_table',
 import numpy as np
 import scipy.linalg as la
 import scipy
-from scipy import angle, pi, exp, sqrt
+from numpy import angle, pi, exp, sqrt
 from types import FunctionType
 from qutip.qobj import Qobj, isket
 from qutip.superoperator import vec2mat_index, mat2vec, vec2mat
@@ -434,7 +434,7 @@ def floquet_state_decomposition(f_states, f_energies, psi):
 
     """
     # [:1,:1][0, 0] patch around scipy 1.3.0 bug
-    return [(f_states[i].dag() * psi).data[:1,:1][0, 0]
+    return [(f_states[i].dag() * psi).data[:1, :1][0, 0]
             for i in np.arange(len(f_energies))]
 
 def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
@@ -621,7 +621,7 @@ def floquet_master_equation_rates(f_modes_0, f_energies, c_op, H, T,
                 for k in range(-kmax, kmax + 1, 1):
                     # [:1,:1][0, 0] patch around scipy 1.3.0 bug
                     X[a, b, k_idx] += (dT / T) * exp(-1j * k * omega * t) * \
-                        (f_modes_t[a].dag() * c_op * f_modes_t[b])[:1,:1][0, 0]
+                        (f_modes_t[a].dag() * c_op * f_modes_t[b])[:1, :1][0, 0]
                     k_idx += 1
 
     Heaviside = lambda x: ((np.sign(x) + 1) / 2.0)
