@@ -37,7 +37,7 @@ mappings, using the builtin Python module multiprocessing.
 __all__ = ['parfor', 'parallel_map', 'serial_map']
 
 from scipy import array
-from multiprocessing import Pool
+import multiprocessing
 from functools import partial
 import os
 import sys
@@ -46,6 +46,12 @@ import qutip.settings as qset
 from qutip.ui.progressbar import BaseProgressBar, TextProgressBar
 
 
+if sys.platform == 'darwin':
+    Pool = multiprocessing.get_context('fork').Pool
+else:
+    Pool = multiprocessing.Pool
+
+
 def _task_wrapper(args):
     try:
         return args[0](*args[1])
@@ -167,7 +173,7 @@ def serial_map(task, values, task_args=tuple(), task_kwargs={}, **kwargs):
         The result list contains the value of
         ``task(value, *task_args, **task_kwargs)`` for each
         value in ``values``.
-    
+
     """
     try:
         progress_bar = kwargs['progress_bar']
@@ -211,8 +217,8 @@ def parallel_map(task, values, task_args=tuple(), task_kwargs={}, **kwargs):
     Returns
     --------
     result : list
-        The result list contains the value of 
-        ``task(value, *task_args, **task_kwargs)`` for 
+        The result list contains the value of
+        ``task(value, *task_args, **task_kwargs)`` for
         each value in ``values``.
 
     """
