@@ -35,7 +35,8 @@ __all__ = ['reshuffle']
 
 import numpy as np
 import scipy.sparse as sp
-from qutip.ptrace import _select
+from qutip.cy.ptrace import _select
+from qutip.cy.spconvert import arr_coo2fast, cy_index_permute
 
 
 def _chunk_dims(dims, order):
@@ -47,91 +48,98 @@ def _chunk_dims(dims, order):
 
 
 def _permute(Q, order):
+    Qcoo = Q.data.tocoo()
+    
     if Q.isket:
-        dims, perm = _perm_inds(Q.dims[0], order)
-        nzs = Q.data.nonzero()[0]
-        wh = np.where(perm == nzs)[0]
-        data = np.ones(len(wh), dtype=int)
-        cols = perm[wh].T[0]
-        perm_matrix = sp.coo_matrix((data, (wh, cols)),
-                                    shape=(Q.shape[0], Q.shape[0]), dtype=int)
-        perm_matrix = perm_matrix.tocsr()
-        return perm_matrix * Q.data, Q.dims
+        cy_index_permute(Qcoo.row,
+                         np.array(Q.dims[0], dtype=np.int32),
+                         np.array(order, dtype=np.int32))
+        
+        new_dims = [[Q.dims[0][i] for i in order], Q.dims[1]]
 
     elif Q.isbra:
-        dims, perm = _perm_inds(Q.dims[0], order)
-        nzs = Q.data.nonzero()[1]
-        wh = np.where(perm == nzs)[0]
-        data = np.ones(len(wh), dtype=int)
-        rows = perm[wh].T[0]
-        perm_matrix = sp.coo_matrix((data, (rows, wh)),
-                                    shape=(Q.shape[1], Q.shape[1]), dtype=int)
-        perm_matrix = perm_matrix.tocsr()
-        return Q.data * perm_matrix, Q.dims
+        cy_index_permute(Qcoo.col,
+                         np.array(Q.dims[1], dtype=np.int32),
+                         np.array(order, dtype=np.int32))
+        
+        new_dims = [Q.dims[0], [Q.dims[1][i] for i in order]]
 
     elif Q.isoper:
-        dims, perm = _perm_inds(Q.dims[0], order)
-        data = np.ones(Q.shape[0], dtype=int)
-        rows = np.arange(Q.shape[0], dtype=int)
-        perm_matrix = sp.coo_matrix((data, (rows, perm.T[0])),
-                                    shape=(Q.shape[1], Q.shape[1]), dtype=int)
-        perm_matrix = perm_matrix.tocsr()
-        dims_part = list(dims[order])
-        dims = [dims_part, dims_part]
-        return (perm_matrix * Q.data) * perm_matrix.T, dims
-
-    elif Q.issuper or Q.isoperket:
-        # For superoperators, we expect order to be something like
+        cy_index_permute(Qcoo.row,
+                         np.array(Q.dims[0], dtype=np.int32),
+                         np.array(order, dtype=np.int32))
+        cy_index_permute(Qcoo.col,
+                         np.array(Q.dims[1], dtype=np.int32),
+                         np.array(order, dtype=np.int32))
+        
+        new_dims = [[Q.dims[0][i] for i in order], [Q.dims[1][i] for i in order]]
+    
+    elif Q.isoperket:
+    	# For superoperators, we expect order to be something like
         # [[0, 2], [1, 3]], which tells us to permute according to
         # [0, 2, 1 ,3], and then group indices according to the length
         # of each sublist.
         # As another example,
         # permuting [[[1, 2, 3], [1, 2, 3]], [[1, 2, 3], [1, 2, 3]]] by
-        # [[0, 3], [1, 4], [2, 6]] should give
+        # [[0, 3], [1, 4], [2, 5]] should give
         # [[[1, 1], [2, 2], [3, 3]], [[1, 1], [2, 2], [3, 3]]].
         #
         # Get the breakout of the left index into dims.
         # Since this is a super, the left index itself breaks into left
         # and right indices, each of which breaks down further.
         # The best way to deal with that here is to flatten dims.
-        flat_order = sum(order, [])
-        q_dims_left = sum(Q.dims[0], [])
-        dims, perm = _perm_inds(q_dims_left, flat_order)
-        dims = dims.flatten()
-
-        data = np.ones(Q.shape[0], dtype=int)
-        rows = np.arange(Q.shape[0], dtype=int)
-
-        perm_matrix = sp.coo_matrix((data, (rows, perm.T[0])),
-                                    shape=(Q.shape[0], Q.shape[0]), dtype=int)
-
-        perm_matrix = perm_matrix.tocsr()
-        dims_part = list(dims[flat_order])
-
+        
+        flat_order = np.array(sum(order, []), dtype=np.int32)
+        q_dims = np.array(sum(Q.dims[0], []), dtype=np.int32)
+        
+        cy_index_permute(Qcoo.row, q_dims, flat_order)
+        
         # Finally, we need to restructure the now-decomposed left index
         # into left and right subindices, so that the overall dims we return
         # are of the form specified by order.
-        dims_part = list(_chunk_dims(dims_part, order))
-        perm_left = (perm_matrix * Q.data)
-        if Q.type == 'operator-ket':
-            return perm_left, [dims_part, [1]]
-        elif Q.type == 'super':
-            return perm_left * perm_matrix.T, [dims_part, dims_part]
+        
+        new_dims = [q_dims[i] for i in flat_order]
+        new_dims = list(_chunk_dims(new_dims, order))
+        new_dims = [new_dims, [1]]
+    
+    elif Q.isoperbra:
+        flat_order = np.array(sum(order, []), dtype=np.int32)
+        q_dims = np.array(sum(Q.dims[1], []), dtype=np.int32)
+        
+        cy_index_permute(Qcoo.col, q_dims, flat_order)
+        
+        new_dims = [q_dims[i] for i in flat_order]
+        new_dims = list(_chunk_dims(new_dims, order))
+        new_dims = [[1], new_dims]
+    
+    elif Q.issuper:
+        flat_order = np.array(sum(order, []), dtype=np.int32)
+        q_dims = np.array(sum(Q.dims[0], []), dtype=np.int32)
+        
+        cy_index_permute(Qcoo.row, q_dims, flat_order)
+        cy_index_permute(Qcoo.col, q_dims, flat_order)
+        
+        new_dims = [q_dims[i] for i in flat_order]
+        new_dims = list(_chunk_dims(new_dims, order))
+        new_dims = [new_dims, new_dims]
+        
     else:
         raise TypeError('Invalid quantum object for permutation.')
+    
+    return arr_coo2fast(Qcoo.data, Qcoo.row, Qcoo.col, Qcoo.shape[0], Qcoo.shape[1]), new_dims
 
 
 def _perm_inds(dims, order):
     """
     Private function giving permuted indices for permute function.
     """
-    dims = np.asarray(dims)
-    order = np.asarray(order)
+    dims = np.asarray(dims,dtype=np.int32)
+    order = np.asarray(order,dtype=np.int32)
     if not np.all(np.sort(order) == np.arange(len(dims))):
         raise ValueError(
             'Requested permutation does not match tensor structure.')
-    sel = _select(order, dims)
-    irev = np.fliplr(sel) - 1
+    sel = _select(order, dims,np.prod(dims))
+    irev = np.fliplr(sel)
     fact = np.append(np.array([1]), np.cumprod(np.flipud(dims)[:-1]))
     fact = fact.reshape(len(fact), 1)
     perm_inds = np.dot(irev, fact)
