@@ -1,6 +1,7 @@
 # This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson,
+#    and the QuTiP Developers.
 #    All rights reserved.
 #
 #    Redistribution and use in source and binary forms, with or without
@@ -35,6 +36,7 @@
 This module calculates the Liouvillian for the dynamics of ensembles of
 identical two-level systems (TLS) in the presence of local and collective
 processes by exploiting permutational symmetry and using the Dicke basis.
+It also allows to characterize nonlinear functions of the density matrix.
 """
 
 # Authors: Nathan Shammah, Shahnawaz Ahmed
@@ -45,17 +47,64 @@ from decimal import Decimal
 
 import numpy as np
 from scipy.integrate import odeint
+from scipy.sparse.linalg import eigsh
+from scipy.special import entr
 from scipy import constants
 from scipy.sparse import dok_matrix, block_diag, lil_matrix
 from qutip.solver import Options, Result
-from qutip import (Qobj, spre, spost, tensor, identity, ket2dm,
-                   vector_to_operator)
+from qutip import (
+    Qobj,
+    spre,
+    spost,
+    tensor,
+    identity,
+    ket2dm,
+    vector_to_operator,
+)
 from qutip import sigmax, sigmay, sigmaz, sigmap, sigmam
+from qutip import entropy_vn
 from qutip.cy.piqs import Dicke as _Dicke
-from qutip.cy.piqs import (jmm1_dictionary, _num_dicke_states,
-                           _num_dicke_ladders, get_blocks, j_min,
-                           m_vals, j_vals)
+from qutip.cy.piqs import (
+    jmm1_dictionary,
+    _num_dicke_states,
+    _num_dicke_ladders,
+    get_blocks,
+    j_min,
+    m_vals,
+    j_vals,
+)
 
+__all__ = [
+    "num_dicke_states",
+    "num_dicke_ladders",
+    "num_tls",
+    "isdiagonal",
+    "dicke_blocks",
+    "dicke_blocks_full",
+    "dicke_function_trace",
+    "purity_dicke",
+    "entropy_vn_dicke",
+    "Dicke",
+    "state_degeneracy",
+    "m_degeneracy",
+    "energy_degeneracy",
+    "ap",
+    "am",
+    "spin_algebra",
+    "jspin",
+    "collapse_uncoupled",
+    "dicke_basis",
+    "dicke",
+    "excited",
+    "superradiant",
+    "css",
+    "ghz",
+    "ground",
+    "identity_uncoupled",
+    "block_matrix",
+    "tau_column",
+    "Pim",
+]
 
 # Functions necessary to generate the Lindbladian/Liouvillian
 def num_dicke_states(N):
@@ -73,6 +122,7 @@ def num_dicke_states(N):
     """
     return _num_dicke_states(N)
 
+
 def num_dicke_ladders(N):
     """Calculate the total number of ladders in the Dicke space.
 
@@ -91,6 +141,7 @@ def num_dicke_ladders(N):
     """
     return _num_dicke_ladders(N)
 
+
 def num_tls(nds):
     """Calculate the number of two-level systems.
 
@@ -106,12 +157,13 @@ def num_tls(nds):
     """
     if np.sqrt(nds).is_integer():
         # N is even
-        N = 2*(np.sqrt(nds)-1)
+        N = 2 * (np.sqrt(nds) - 1)
     else:
         # N is odd
-        N = 2*(np.sqrt(nds + 1/4)-1)
+        N = 2 * (np.sqrt(nds + 1 / 4) - 1)
     return int(N)
 
+
 def isdiagonal(mat):
     """
     Check if the input matrix is diagonal.
@@ -132,6 +184,165 @@ def isdiagonal(mat):
     return np.all(mat == np.diag(np.diagonal(mat)))
 
 
+# nonlinear functions of the density matrix
+def dicke_blocks(rho):
+    """Create the list of blocks for block-diagonal density matrix in the Dicke basis.
+
+    Parameters
+    ----------
+    rho : :class:`qutip.Qobj`
+        A 2D block-diagonal matrix of ones with dimension (nds,nds),
+        where nds is the number of Dicke states for N two-level
+        systems.
+
+    Returns
+    -------
+    square_blocks: list of np.array
+        Give back the blocks list.
+
+    """
+    shape_dimension = rho.shape[0]
+    N = num_tls(shape_dimension)
+    ladders = num_dicke_ladders(N)
+    # create a list with the sizes of the blocks, in order
+    blocks_dimensions = int(N / 2 + 1 - 0.5 * (N % 2))
+    blocks_list = [
+        (2 * (i + 1 * (N % 2)) + 1 * ((N + 1) % 2))
+        for i in range(blocks_dimensions)
+    ]
+    blocks_list = np.flip(blocks_list, 0)
+    # create a list with each block matrix as element
+    square_blocks = []
+    block_position = 0
+    for block_size in blocks_list:
+        start_m = block_position
+        end_m = block_position + block_size
+        square_block = rho[start_m:end_m, start_m:end_m]
+        block_position = block_position + block_size
+        square_blocks.append(square_block)
+
+    return square_blocks
+
+
+def dicke_blocks_full(rho):
+    """Give the full (2^N-dimensional) list of blocks for a Dicke-basis matrix.
+
+    Parameters
+    ----------
+    rho : :class:`qutip.Qobj`
+        A 2D block-diagonal matrix of ones with dimension (nds,nds),
+        where nds is the number of Dicke states for N two-level
+        systems.
+
+    Returns
+    -------
+    full_blocks : list
+        The list of blocks expanded in the 2^N space for N qubits.
+
+    """
+    shape_dimension = rho.shape[0]
+    N = num_tls(shape_dimension)
+    ladders = num_dicke_ladders(N)
+    # create a list with the sizes of the blocks, in order
+    blocks_dimensions = int(N / 2 + 1 - 0.5 * (N % 2))
+    blocks_list = [
+        (2 * (i + 1 * (N % 2)) + 1 * ((N + 1) % 2))
+        for i in range(blocks_dimensions)
+    ]
+    blocks_list = np.flip(blocks_list, 0)
+    # create a list with each block matrix as element
+    full_blocks = []
+    k = 0
+    block_position = 0
+    for block_size in blocks_list:
+        start_m = block_position
+        end_m = block_position + block_size
+        square_block = rho[start_m:end_m, start_m:end_m]
+        block_position = block_position + block_size
+        j = N / 2 - k
+        djn = state_degeneracy(N, j)
+        for block_counter in range(0, djn):
+            full_blocks.append(square_block / djn)  # preserve trace
+        k = k + 1
+    return full_blocks
+
+
+def dicke_function_trace(f, rho):
+    """Calculate the trace of a function on a Dicke density matrix.
+    Parameters
+    ----------
+    f : function
+        A Taylor-expandable function of `rho`.
+
+    rho : :class:`qutip.Qobj`
+        A density matrix in the Dicke basis.
+
+    Returns
+    -------
+    res : float
+        Trace of a nonlinear function on `rho`.
+    """
+    N = num_tls(rho.shape[0])
+    blocks = dicke_blocks(rho)
+    block_f = []
+    degen_blocks = np.flip(j_vals(N))
+    state_degeneracies = []
+    for j in degen_blocks:
+        dj = state_degeneracy(N, j)
+        state_degeneracies.append(dj)
+    eigenvals_degeneracy = []
+    deg = []
+    for i, block in enumerate(blocks):
+        dj = state_degeneracies[i]
+        normalized_block = block / dj
+        eigenvals_block = eigsh(normalized_block, return_eigenvectors=False)
+        for val in eigenvals_block:
+            eigenvals_degeneracy.append(val)
+            deg.append(dj)
+
+    eigenvalue = np.array(eigenvals_degeneracy)
+    function_array = f(eigenvalue) * deg
+    return sum(function_array)
+
+
+def entropy_vn_dicke(rho):
+    """Von Neumann Entropy of a Dicke-basis density matrix.
+
+    Parameters
+    ----------
+    rho : :class:`qutip.Qobj`
+        A 2D block-diagonal matrix of ones with dimension (nds,nds),
+        where nds is the number of Dicke states for N two-level
+        systems.
+
+    Returns
+    -------
+    entropy_dm: float
+        Entropy. Use degeneracy to multiply each block.
+
+    """
+    return dicke_function_trace(entr, rho)
+
+
+def purity_dicke(rho):
+    """Calculate purity of a density matrix in the Dicke basis.
+    It accounts for the degenerate blocks in the density matrix.
+
+    Parameters
+    ----------
+    rho : :class:`qutip.Qobj`
+        Density matrix in the Dicke basis of qutip.piqs.jspin(N), for N spins.
+
+    Returns
+    -------
+    purity : float
+        The purity of the quantum state.
+        It's 1 for pure states, 0<=purity<1 for mixed states.
+    """
+    f = lambda x: x * x
+    return dicke_function_trace(f, rho)
+
+
 class Dicke(object):
     """The Dicke class which builds the Lindbladian and Liouvillian matrix.
 
@@ -226,10 +437,18 @@ class Dicke(object):
         The shape of the Hilbert space in the Dicke or uncoupled basis.
         default: (nds, nds).
     """
-    def __init__(self, N, hamiltonian=None,
-                 emission=0., dephasing=0., pumping=0.,
-                 collective_emission=0., collective_dephasing=0.,
-                 collective_pumping=0.):
+
+    def __init__(
+        self,
+        N,
+        hamiltonian=None,
+        emission=0.0,
+        dephasing=0.0,
+        pumping=0.0,
+        collective_emission=0.0,
+        collective_dephasing=0.0,
+        collective_pumping=0.0,
+    ):
         self.N = N
         self.hamiltonian = hamiltonian
         self.emission = emission
@@ -247,8 +466,9 @@ class Dicke(object):
         string.append("N = {}".format(self.N))
         string.append("Hilbert space dim = {}".format(self.dshape))
         string.append("Number of Dicke states = {}".format(self.nds))
-        string.append("Liouvillian space dim = {}".format((self.nds**2,
-                                                           self.nds**2)))
+        string.append(
+            "Liouvillian space dim = {}".format((self.nds ** 2, self.nds ** 2))
+        )
         if self.emission != 0:
             string.append("emission = {}".format(self.emission))
         if self.dephasing != 0:
@@ -257,13 +477,16 @@ class Dicke(object):
             string.append("pumping = {}".format(self.pumping))
         if self.collective_emission != 0:
             string.append(
-                "collective_emission = {}".format(self.collective_emission))
+                "collective_emission = {}".format(self.collective_emission)
+            )
         if self.collective_dephasing != 0:
             string.append(
-                "collective_dephasing = {}".format(self.collective_dephasing))
+                "collective_dephasing = {}".format(self.collective_dephasing)
+            )
         if self.collective_pumping != 0:
             string.append(
-                "collective_pumping = {}".format(self.collective_pumping))
+                "collective_pumping = {}".format(self.collective_pumping)
+            )
         return "\n".join(string)
 
     def lindbladian(self):
@@ -274,13 +497,15 @@ class Dicke(object):
         lindbladian : :class:`qutip.Qobj`
             The Lindbladian matrix as a `qutip.Qobj`.
         """
-        cythonized_dicke = _Dicke(int(self.N),
-                                  float(self.emission),
-                                  float(self.dephasing),
-                                  float(self.pumping),
-                                  float(self.collective_emission),
-                                  float(self.collective_dephasing),
-                                  float(self.collective_pumping))
+        cythonized_dicke = _Dicke(
+            int(self.N),
+            float(self.emission),
+            float(self.dephasing),
+            float(self.pumping),
+            float(self.collective_emission),
+            float(self.collective_dephasing),
+            float(self.collective_pumping),
+        )
         return cythonized_dicke.lindbladian()
 
     def liouvillian(self):
@@ -297,8 +522,9 @@ class Dicke(object):
 
         else:
             hamiltonian = self.hamiltonian
-            hamiltonian_superoperator = - 1j * \
-                spre(hamiltonian) + 1j * spost(hamiltonian)
+            hamiltonian_superoperator = -1j * spre(hamiltonian) + 1j * spost(
+                hamiltonian
+            )
             liouv = lindblad + hamiltonian_superoperator
         return liouv
 
@@ -340,9 +566,15 @@ class Dicke(object):
             msg = "Initial density matrix should be diagonal."
             raise ValueError(msg)
 
-        pim = Pim(self.N, self.emission, self.dephasing, self.pumping,
-                  self.collective_emission, self.collective_pumping,
-                  self.collective_dephasing)
+        pim = Pim(
+            self.N,
+            self.emission,
+            self.dephasing,
+            self.pumping,
+            self.collective_emission,
+            self.collective_pumping,
+            self.collective_dephasing,
+        )
         result = pim.solve(initial_state, tlist, options=None)
         return result
 
@@ -357,13 +589,15 @@ class Dicke(object):
         ce = self.collective_emission
         cd = self.collective_dephasing
         cp = self.collective_pumping
-        c_ops_list = collapse_uncoupled(N=self.N,
-                                        emission=self.emission,
-                                        dephasing=self.dephasing,
-                                        pumping=self.pumping,
-                                        collective_emission=ce,
-                                        collective_dephasing=cd,
-                                        collective_pumping=cp)
+        c_ops_list = collapse_uncoupled(
+            N=self.N,
+            emission=self.emission,
+            dephasing=self.dephasing,
+            pumping=self.pumping,
+            collective_emission=ce,
+            collective_dephasing=cd,
+            collective_pumping=cp,
+        )
         return c_ops_list
 
     def coefficient_matrix(self):
@@ -376,13 +610,15 @@ class Dicke(object):
             p is the vector of the diagonal matrix elements
             of the density matrix rho in the Dicke basis.
         """
-        diagonal_system = Pim(N=self.N,
-                              emission=self.emission,
-                              dephasing=self.dephasing,
-                              pumping=self.pumping,
-                              collective_emission=self.collective_emission,
-                              collective_dephasing=self.collective_dephasing,
-                              collective_pumping=self.collective_pumping)
+        diagonal_system = Pim(
+            N=self.N,
+            emission=self.emission,
+            dephasing=self.dephasing,
+            pumping=self.pumping,
+            collective_emission=self.collective_emission,
+            collective_dephasing=self.collective_dephasing,
+            collective_pumping=self.collective_pumping,
+        )
         coef_matrix = diagonal_system.coefficient_matrix()
         return coef_matrix
 
@@ -409,11 +645,12 @@ def energy_degeneracy(N, m):
         The energy degeneracy
     """
     numerator = Decimal(factorial(N))
-    d1 = Decimal(factorial(N/2 + m))
-    d2 = Decimal(factorial(N/2 - m))
-    degeneracy = numerator/(d1 * d2)
+    d1 = Decimal(factorial(N / 2 + m))
+    d2 = Decimal(factorial(N / 2 - m))
+    degeneracy = numerator / (d1 * d2)
     return int(degeneracy)
 
+
 def state_degeneracy(N, j):
     """Calculate the degeneracy of the Dicke state.
 
@@ -437,13 +674,14 @@ def state_degeneracy(N, j):
     """
     if j < 0:
         raise ValueError("j value should be >= 0")
-    numerator = Decimal(factorial(N)) * Decimal(2*j + 1)
-    denominator_1 = Decimal(factorial(N/2 + j + 1))
-    denominator_2 = Decimal(factorial(N/2 - j))
-    degeneracy = numerator/(denominator_1 * denominator_2)
+    numerator = Decimal(factorial(N)) * Decimal(2 * j + 1)
+    denominator_1 = Decimal(factorial(N / 2 + j + 1))
+    denominator_2 = Decimal(factorial(N / 2 - j))
+    degeneracy = numerator / (denominator_1 * denominator_2)
     degeneracy = int(np.round(float(degeneracy)))
     return degeneracy
 
+
 def m_degeneracy(N, m):
     """Calculate the number of Dicke states :math:`|j, m\\rangle` with
     same energy.
@@ -468,9 +706,10 @@ def m_degeneracy(N, m):
         e = "m value is incorrect for this N."
         e += " Minimum m value can be {}".format(-maxj)
         raise ValueError(e)
-    degeneracy = N/2 + 1 - abs(m)
+    degeneracy = N / 2 + 1 - abs(m)
     return int(degeneracy)
 
+
 def ap(j, m):
     """Calculate the coefficient `ap` by applying J_+ |j, m>.
 
@@ -487,9 +726,10 @@ def ap(j, m):
     a_plus: float
         The value of :math:`a_{+}`.
     """
-    a_plus = np.sqrt((j-m) * (j+m+1))
+    a_plus = np.sqrt((j - m) * (j + m + 1))
     return a_plus
 
+
 def am(j, m):
     """Calculate the operator `am` used later.
 
@@ -508,9 +748,10 @@ def am(j, m):
     a_minus: float
         The value of :math:`a_{-}`.
     """
-    a_minus = np.sqrt((j+m) * (j-m+1))
+    a_minus = np.sqrt((j + m) * (j - m + 1))
     return a_minus
 
+
 def spin_algebra(N, op=None):
     """Create the list [sx, sy, sz] with the spin operators.
 
@@ -573,18 +814,19 @@ def spin_algebra(N, op=None):
 
     if not op:
         return spin_operators
-    elif op == 'x':
+    elif op == "x":
         return sx
-    elif op == 'y':
+    elif op == "y":
         return sy
-    elif op == 'z':
+    elif op == "z":
         return sz
-    elif op == '+':
+    elif op == "+":
         return sp
-    elif op == '-':
+    elif op == "-":
         return sm
     else:
-        raise TypeError('Invalid type')
+        raise TypeError("Invalid type")
+
 
 def _jspin_uncoupled(N, op=None):
     """Construct the the collective spin algebra in the uncoupled basis.
@@ -625,18 +867,19 @@ def _jspin_uncoupled(N, op=None):
 
     if not op:
         return collective_operators
-    elif op == 'x':
+    elif op == "x":
         return jx
-    elif op == 'y':
+    elif op == "y":
         return jy
-    elif op == 'z':
+    elif op == "z":
         return jz
-    elif op == '+':
+    elif op == "+":
         return jp
-    elif op == '-':
+    elif op == "-":
         return jm
     else:
-        raise TypeError('Invalid type')
+        raise TypeError("Invalid type")
+
 
 def jspin(N, op=None, basis="dicke"):
     """
@@ -671,25 +914,25 @@ def jspin(N, op=None, basis="dicke"):
 
     nds = num_dicke_states(N)
     num_ladders = num_dicke_ladders(N)
-    jz_operator = dok_matrix((nds, nds), dtype=np.complex)
-    jp_operator = dok_matrix((nds, nds), dtype=np.complex)
-    jm_operator = dok_matrix((nds, nds), dtype=np.complex)
+    jz_operator = dok_matrix((nds, nds), dtype=np.complex128)
+    jp_operator = dok_matrix((nds, nds), dtype=np.complex128)
+    jm_operator = dok_matrix((nds, nds), dtype=np.complex128)
     s = 0
 
     for k in range(0, num_ladders):
         j = 0.5 * N - k
-        mmax = int(2*j + 1)
+        mmax = int(2 * j + 1)
         for i in range(0, mmax):
-            m = j-i
+            m = j - i
             jz_operator[s, s] = m
-            if (s+1) in range(0, nds):
-                jp_operator[s, s+1] = ap(j, m-1)
-            if (s-1) in range(0, nds):
-                jm_operator[s, s-1] = am(j, m+1)
-            s = s+1
+            if (s + 1) in range(0, nds):
+                jp_operator[s, s + 1] = ap(j, m - 1)
+            if (s - 1) in range(0, nds):
+                jm_operator[s, s - 1] = am(j, m + 1)
+            s = s + 1
 
-    jx_operator = 1/2 * (jp_operator+jm_operator)
-    jy_operator = 1j/2 * (jm_operator-jp_operator)
+    jx_operator = 1 / 2 * (jp_operator + jm_operator)
+    jy_operator = 1j / 2 * (jm_operator - jp_operator)
     jx = Qobj(jx_operator)
     jy = Qobj(jy_operator)
     jz = Qobj(jz_operator)
@@ -698,22 +941,29 @@ def jspin(N, op=None, basis="dicke"):
 
     if not op:
         return [jx, jy, jz]
-    if op == '+':
+    if op == "+":
         return jp
-    elif op == '-':
+    elif op == "-":
         return jm
-    elif op == 'x':
+    elif op == "x":
         return jx
-    elif op == 'y':
+    elif op == "y":
         return jy
-    elif op == 'z':
+    elif op == "z":
         return jz
     else:
-        raise TypeError('Invalid type')
+        raise TypeError("Invalid type")
 
-def collapse_uncoupled(N, emission=0., dephasing=0., pumping=0.,
-              collective_emission=0., collective_dephasing=0.,
-              collective_pumping=0.):
+
+def collapse_uncoupled(
+    N,
+    emission=0.0,
+    dephasing=0.0,
+    pumping=0.0,
+    collective_emission=0.0,
+    collective_dephasing=0.0,
+    collective_pumping=0.0,
+):
     """
     Create the collapse operators (c_ops) of the Lindbladian in the
     uncoupled basis
@@ -773,8 +1023,10 @@ def collapse_uncoupled(N, emission=0., dephasing=0., pumping=0.,
     [sx, sy, sz] = spin_algebra(N)
     sp, sm = spin_algebra(N, "+"), spin_algebra(N, "-")
     [jx, jy, jz] = jspin(N, basis="uncoupled")
-    jp, jm = (jspin(N, "+", basis = "uncoupled"),
-              jspin(N, "-", basis="uncoupled"))
+    jp, jm = (
+        jspin(N, "+", basis="uncoupled"),
+        jspin(N, "-", basis="uncoupled"),
+    )
 
     c_ops = []
 
@@ -801,6 +1053,7 @@ def collapse_uncoupled(N, emission=0., dephasing=0., pumping=0.,
 
     return c_ops
 
+
 # State definitions in the Dicke basis with an option for basis transformation
 def dicke_basis(N, jmm1=None):
     """
@@ -841,6 +1094,7 @@ def dicke_basis(N, jmm1=None):
         rho[i, k] = jmm1[key]
     return Qobj(rho)
 
+
 def dicke(N, j, m):
     """
     Generate a Dicke state as a pure density matrix in the Dicke basis.
@@ -873,9 +1127,10 @@ def dicke(N, j, m):
     jmm1_dict = jmm1_dictionary(N)[1]
 
     i, k = jmm1_dict[(j, m, m)]
-    rho[i, k] = 1.
+    rho[i, k] = 1.0
     return Qobj(rho)
 
+
 # Uncoupled states in the full Hilbert space. These are returned with the
 # choice of the keyword argument `basis="uncoupled"` in the state functions.
 def _uncoupled_excited(N):
@@ -896,9 +1151,10 @@ def _uncoupled_excited(N):
     N = int(N)
     jz = jspin(N, "z", basis="uncoupled")
     en, vn = jz.eigenstates()
-    psi0 = vn[2**N - 1]
+    psi0 = vn[2 ** N - 1]
     return ket2dm(psi0)
 
+
 def _uncoupled_superradiant(N):
     """
     Generate the density matrix of a superradiant state in the full
@@ -918,9 +1174,10 @@ def _uncoupled_superradiant(N):
     N = int(N)
     jz = jspin(N, "z", basis="uncoupled")
     en, vn = jz.eigenstates()
-    psi0 = vn[2**N - (N+1)]
+    psi0 = vn[2 ** N - (N + 1)]
     return ket2dm(psi0)
 
+
 def _uncoupled_ground(N):
     """
     Generate the density matrix of the ground state in the full\
@@ -942,6 +1199,7 @@ def _uncoupled_ground(N):
     psi0 = vn[0]
     return ket2dm(psi0)
 
+
 def _uncoupled_ghz(N):
     """
     Generate the density matrix of the GHZ state in the full 2^N
@@ -958,16 +1216,17 @@ def _uncoupled_ghz(N):
         The density matrix for the GHZ state in the full Hilbert space.
     """
     N = int(N)
-    rho = np.zeros((2**N, 2**N))
-    rho[0, 0] = 1/2
-    rho[2**N - 1, 0] = 1/2
-    rho[0, 2**N - 1] = 1/2
-    rho[2**N - 1, 2**N - 1] = 1/2
+    rho = np.zeros((2 ** N, 2 ** N))
+    rho[0, 0] = 1 / 2
+    rho[2 ** N - 1, 0] = 1 / 2
+    rho[0, 2 ** N - 1] = 1 / 2
+    rho[2 ** N - 1, 2 ** N - 1] = 1 / 2
     spin_dim = [2 for i in range(0, N)]
     spins_dims = list((spin_dim, spin_dim))
     rho = Qobj(rho, dims=spins_dims)
     return rho
 
+
 def _uncoupled_css(N, a, b):
     """
     Generate the density matrix of the CSS state in the full 2^N
@@ -1013,12 +1272,13 @@ def _uncoupled_css(N, a, b):
     # |+><+| = Prod_(i=1)^N |CSS>_i<CSS|_i
     for i in range(1, N):
         rho[i] = rho[0].permute(b[i])
-    identity_i = Qobj(np.eye(2**N), dims=rho[0].dims, shape=rho[0].shape)
+    identity_i = Qobj(np.eye(2 ** N), dims=rho[0].dims, shape=rho[0].shape)
     rho_tot = identity_i
     for i in range(0, N):
         rho_tot = rho_tot * rho[i]
     return rho_tot
 
+
 def excited(N, basis="dicke"):
     """
     Generate the density matrix for the excited state.
@@ -1044,9 +1304,10 @@ def excited(N, basis="dicke"):
         state = _uncoupled_excited(N)
         return state
 
-    jmm1 = {(N/2, N/2, N/2): 1}
+    jmm1 = {(N / 2, N / 2, N / 2): 1}
     return dicke_basis(N, jmm1)
 
+
 def superradiant(N, basis="dicke"):
     """
     Generate the density matrix of the superradiant state.
@@ -1073,14 +1334,20 @@ def superradiant(N, basis="dicke"):
         return state
 
     if N % 2 == 0:
-        jmm1 = {(N/2, 0, 0): 1.}
+        jmm1 = {(N / 2, 0, 0): 1.0}
         return dicke_basis(N, jmm1)
     else:
-        jmm1 = {(N/2, 0.5, 0.5): 1.}
+        jmm1 = {(N / 2, 0.5, 0.5): 1.0}
     return dicke_basis(N, jmm1)
 
-def css(N, x=1/np.sqrt(2), y=1/np.sqrt(2),
-        basis="dicke", coordinates="cartesian"):
+
+def css(
+    N,
+    x=1 / np.sqrt(2),
+    y=1 / np.sqrt(2),
+    basis="dicke",
+    coordinates="cartesian",
+):
     """
     Generate the density matrix of the Coherent Spin State (CSS).
 
@@ -1123,25 +1390,32 @@ def css(N, x=1/np.sqrt(2), y=1/np.sqrt(2),
         return _uncoupled_css(N, a, b)
     nds = num_dicke_states(N)
     num_ladders = num_dicke_ladders(N)
-    rho = dok_matrix((nds, nds), dtype=np.complex)
+    rho = dok_matrix((nds, nds), dtype=np.complex128)
 
     # loop in the allowed matrix elements
     jmm1_dict = jmm1_dictionary(N)[1]
 
-    j = 0.5*N
-    mmax = int(2*j + 1)
+    j = 0.5 * N
+    mmax = int(2 * j + 1)
     for i in range(0, mmax):
-        m = j-i
-        psi_m = np.sqrt(float(energy_degeneracy(N, m))) * \
-            a**(N*0.5 + m) * b**(N*0.5 - m)
+        m = j - i
+        psi_m = (
+            np.sqrt(float(energy_degeneracy(N, m)))
+            * a ** (N * 0.5 + m)
+            * b ** (N * 0.5 - m)
+        )
         for i1 in range(0, mmax):
             m1 = j - i1
             row_column = jmm1_dict[(j, m, m1)]
-            psi_m1 = np.sqrt(float(energy_degeneracy(N, m1))) * \
-                np.conj(a)**(N*0.5 + m1) * np.conj(b)**(N*0.5 - m1)
-            rho[row_column] = psi_m*psi_m1
+            psi_m1 = (
+                np.sqrt(float(energy_degeneracy(N, m1)))
+                * np.conj(a) ** (N * 0.5 + m1)
+                * np.conj(b) ** (N * 0.5 - m1)
+            )
+            rho[row_column] = psi_m * psi_m1
     return Qobj(rho)
 
+
 def ghz(N, basis="dicke"):
     """
     Generate the density matrix of the GHZ state.
@@ -1165,13 +1439,14 @@ def ghz(N, basis="dicke"):
     if basis == "uncoupled":
         return _uncoupled_ghz(N)
     nds = _num_dicke_states(N)
-    rho = dok_matrix((nds, nds), dtype=np.complex)
-    rho[0, 0] = 1/2
-    rho[N, N] = 1/2
-    rho[N, 0] = 1/2
-    rho[0, N] = 1/2
+    rho = dok_matrix((nds, nds), dtype=np.complex128)
+    rho[0, 0] = 1 / 2
+    rho[N, N] = 1 / 2
+    rho[N, 0] = 1 / 2
+    rho[0, N] = 1 / 2
     return Qobj(rho)
 
+
 def ground(N, basis="dicke"):
     """
     Generate the density matrix of the ground state.
@@ -1197,10 +1472,11 @@ def ground(N, basis="dicke"):
         state = _uncoupled_ground(N)
         return state
     nds = _num_dicke_states(N)
-    rho = dok_matrix((nds, nds), dtype=np.complex)
+    rho = dok_matrix((nds, nds), dtype=np.complex128)
     rho[N, N] = 1
     return Qobj(rho)
 
+
 def identity_uncoupled(N):
     """
     Generate the identity in a :math:`2^N`-dimensional Hilbert space.
@@ -1218,43 +1494,53 @@ def identity_uncoupled(N):
         The identity matrix.
     """
     N = int(N)
-    rho = np.zeros((2**N, 2**N))
-    for i in range(0, 2**N):
+    rho = np.zeros((2 ** N, 2 ** N))
+    for i in range(0, 2 ** N):
         rho[i, i] = 1
     spin_dim = [2 for i in range(0, N)]
     spins_dims = list((spin_dim, spin_dim))
     identity = Qobj(rho, dims=spins_dims)
     return identity
 
-def block_matrix(N):
+
+def block_matrix(N, elements="ones"):
     """Construct the block-diagonal matrix for the Dicke basis.
 
     Parameters
     ----------
-    N: int
+    N : int
         Number of two-level systems.
+    elements : str {'ones' (default),'degeneracy'}
 
     Returns
     -------
-    block_matr: ndarray
-        A 2D block-diagonal matrix of ones with dimension (nds,nds),
+    block_matr : ndarray
+        A 2D block-diagonal matrix with dimension (nds,nds),
         where nds is the number of Dicke states for N two-level
-        systems.
+        systems. Filled with ones or the value of degeneracy
+        at each matrix element.
     """
-    nds = _num_dicke_states(N)
     # create a list with the sizes of the blocks, in order
-    blocks_dimensions = int(N/2 + 1 - 0.5*(N % 2))
-    blocks_list = [(2 * (i+1 * (N % 2)) + 1*((N+1) % 2))
-                   for i in range(blocks_dimensions)]
+    blocks_dimensions = int(N / 2 + 1 - 0.5 * (N % 2))
+    blocks_list = [
+        (2 * (i + 1 * (N % 2)) + 1 * ((N + 1) % 2))
+        for i in range(blocks_dimensions)
+    ]
     blocks_list = np.flip(blocks_list, 0)
     # create a list with each block matrix as element
     square_blocks = []
     k = 0
     for i in blocks_list:
-        square_blocks.append(np.ones((i, i)))
+        if elements == "ones":
+            square_blocks.append(np.ones((i, i)))
+        elif elements == "degeneracy":
+            j = N / 2 - k
+            dj = state_degeneracy(N, j)
+            square_blocks.append(dj * np.ones((i, i)))
         k = k + 1
     return block_diag(square_blocks)
 
+
 # ============================================================================
 # Adding a faster version to make a Permutational Invariant matrix
 # ============================================================================
@@ -1277,15 +1563,17 @@ def tau_column(tau, k, j):
     """
     # In the notes, we indexed from k = 1, here we do it from k = 0
     k = k + 1
-    mapping = {"tau3": k-(2 * j + 3),
-               "tau2": k-1,
-               "tau4": k+(2 * j - 1),
-               "tau5": k-(2 * j + 2),
-               "tau1": k,
-               "tau6": k+(2 * j),
-               "tau7": k-(2 * j + 1),
-               "tau8": k+1,
-               "tau9": k+(2 * j + 1)}
+    mapping = {
+        "tau3": k - (2 * j + 3),
+        "tau2": k - 1,
+        "tau4": k + (2 * j - 1),
+        "tau5": k - (2 * j + 2),
+        "tau1": k,
+        "tau6": k + (2 * j),
+        "tau7": k - (2 * j + 1),
+        "tau8": k + 1,
+        "tau9": k + (2 * j + 1),
+    }
     # we need to decrement k again as indexing is from 0
     return int(mapping[tau] - 1)
 
@@ -1361,9 +1649,17 @@ class Pim(object):
         A nested dictionary of the structure {row: {col: val}} which holds
         non zero elements of the matrix M
     """
-    def __init__(self, N, emission=0., dephasing=0, pumping=0,
-                 collective_emission=0, collective_pumping=0,
-                 collective_dephasing=0):
+
+    def __init__(
+        self,
+        N,
+        emission=0.0,
+        dephasing=0,
+        pumping=0,
+        collective_emission=0,
+        collective_pumping=0,
+        collective_dephasing=0,
+    ):
         self.N = N
         self.emission = emission
         self.dephasing = dephasing
@@ -1389,16 +1685,16 @@ class Pim(object):
         cols = 0
 
         if (self.N % 2) == 0:
-            cols = int(self.N/2 + 1)
+            cols = int(self.N / 2 + 1)
         else:
-            cols = int(self.N/2 + 1/2)
+            cols = int(self.N / 2 + 1 / 2)
         if (dicke_row > rows) or (dicke_row < 0):
-            return (False)
-        if (dicke_col > cols) or (dicke_col < 0):
-            return (False)
-        if (dicke_row < int(rows/2)) and (dicke_col > dicke_row):
             return False
-        if (dicke_row >= int(rows/2)) and (rows - dicke_row <= dicke_col):
+        if (dicke_col > cols) or (dicke_col < 0):
+            return False
+        if (dicke_row < int(rows / 2)) and (dicke_col > dicke_row):
+            return False
+        if (dicke_row >= int(rows / 2)) and (rows - dicke_row <= dicke_col):
             return False
         else:
             return True
@@ -1421,16 +1717,27 @@ class Pim(object):
             A dictionary of key, val as {tau: value} consisting of the valid
             taus for this row and column of the Dicke space element.
         """
-        tau_functions = [self.tau3, self.tau2, self.tau4,
-                         self.tau5, self.tau1, self.tau6,
-                         self.tau7, self.tau8, self.tau9]
+        tau_functions = [
+            self.tau3,
+            self.tau2,
+            self.tau4,
+            self.tau5,
+            self.tau1,
+            self.tau6,
+            self.tau7,
+            self.tau8,
+            self.tau9,
+        ]
         N = self.N
         if self.isdicke(dicke_row, dicke_col) is False:
             return False
         # The 3x3 sub matrix surrounding the Dicke space element to
         # run the tau functions
-        indices = [(dicke_row + x, dicke_col + y) for x in range(-1, 2)
-                   for y in range(-1, 2)]
+        indices = [
+            (dicke_row + x, dicke_col + y)
+            for x in range(-1, 2)
+            for y in range(-1, 2)
+        ]
         taus = {}
         for idx, tau in zip(indices, tau_functions):
             if self.isdicke(idx[0], idx[1]):
@@ -1453,9 +1760,9 @@ class Pim(object):
             The j and m values.
         """
         N = self.N
-        j = N/2 - dicke_col
-        m = N/2 - dicke_row
-        return(j, m)
+        j = N / 2 - dicke_col
+        m = N / 2 - dicke_row
+        return (j, m)
 
     def calculate_k(self, dicke_row, dicke_col):
         """
@@ -1475,8 +1782,10 @@ class Pim(object):
         if dicke_row == 0:
             k = dicke_col
         else:
-            k = int(((dicke_col)/2) * (2 * (N + 1) - 2 * (dicke_col - 1)) +
-                                      (dicke_row - (dicke_col)))
+            k = int(
+                ((dicke_col) / 2) * (2 * (N + 1) - 2 * (dicke_col - 1))
+                + (dicke_row - (dicke_col))
+            )
         return k
 
     def coefficient_matrix(self):
@@ -1493,9 +1802,9 @@ class Pim(object):
 
         sparse_M = lil_matrix((nds, nds), dtype=float)
         if (self.N % 2) == 0:
-            cols = int(self.N/2 + 1)
+            cols = int(self.N / 2 + 1)
         else:
-            cols = int(self.N/2 + 1/2)
+            cols = int(self.N / 2 + 1 / 2)
         for (dicke_row, dicke_col) in np.ndindex(rows, cols):
             if self.isdicke(dicke_row, dicke_col):
                 k = int(self.calculate_k(dicke_row, dicke_col))
@@ -1538,13 +1847,13 @@ class Pim(object):
         yCP = self.collective_pumping
         N = float(self.N)
         spontaneous = yS * (1 + j - m) * (j + m)
-        losses = yL * (N/2 + m)
-        pump = yP * (N/2 - m)
+        losses = yL * (N / 2 + m)
+        pump = yP * (N / 2 - m)
         collective_pump = yCP * (1 + j + m) * (j - m)
         if j == 0:
-            dephase = yD * N/4
+            dephase = yD * N / 4
         else:
-            dephase = yD * (N/4 - m**2 * ((1 + N/2)/(2 * j * (j+1))))
+            dephase = yD * (N / 4 - m ** 2 * ((1 + N / 2) / (2 * j * (j + 1))))
         t1 = spontaneous + losses + pump + dephase + collective_pump
         return -t1
 
@@ -1556,7 +1865,9 @@ class Pim(object):
         yL = self.emission
         N = float(self.N)
         spontaneous = yS * (1 + j - m) * (j + m)
-        losses = yL * (((N/2 + 1) * (j - m + 1) * (j + m))/(2 * j * (j+1)))
+        losses = yL * (
+            ((N / 2 + 1) * (j - m + 1) * (j + m)) / (2 * j * (j + 1))
+        )
         t2 = spontaneous + losses
         return t2
 
@@ -1566,9 +1877,9 @@ class Pim(object):
         """
         yL = self.emission
         N = float(self.N)
-        num = (j + m - 1) * (j + m) * (j + 1 + N/2)
+        num = (j + m - 1) * (j + m) * (j + 1 + N / 2)
         den = 2 * j * (2 * j + 1)
-        t3 = yL * (num/den)
+        t3 = yL * (num / den)
         return t3
 
     def tau4(self, j, m):
@@ -1577,9 +1888,9 @@ class Pim(object):
         """
         yL = self.emission
         N = float(self.N)
-        num = (j - m + 1) * (j - m + 2) * (N/2 - j)
+        num = (j - m + 1) * (j - m + 2) * (N / 2 - j)
         den = 2 * (j + 1) * (2 * j + 1)
-        t4 = yL * (num/den)
+        t4 = yL * (num / den)
         return t4
 
     def tau5(self, j, m):
@@ -1588,9 +1899,9 @@ class Pim(object):
         """
         yD = self.dephasing
         N = float(self.N)
-        num = (j - m) * (j + m) * (j + 1 + N/2)
+        num = (j - m) * (j + m) * (j + 1 + N / 2)
         den = 2 * j * (2 * j + 1)
-        t5 = yD * (num/den)
+        t5 = yD * (num / den)
         return t5
 
     def tau6(self, j, m):
@@ -1599,9 +1910,9 @@ class Pim(object):
         """
         yD = self.dephasing
         N = float(self.N)
-        num = (j - m + 1) * (j + m + 1) * (N/2 - j)
+        num = (j - m + 1) * (j + m + 1) * (N / 2 - j)
         den = 2 * (j + 1) * (2 * j + 1)
-        t6 = yD * (num/den)
+        t6 = yD * (num / den)
         return t6
 
     def tau7(self, j, m):
@@ -1610,9 +1921,9 @@ class Pim(object):
         """
         yP = self.pumping
         N = float(self.N)
-        num = (j - m - 1) * (j - m) * (j + 1 + N/2)
+        num = (j - m - 1) * (j - m) * (j + 1 + N / 2)
         den = 2 * j * (2 * j + 1)
-        t7 = yP * (float(num)/den)
+        t7 = yP * (float(num) / den)
         return t7
 
     def tau8(self, j, m):
@@ -1623,10 +1934,10 @@ class Pim(object):
         yCP = self.collective_pumping
         N = float(self.N)
 
-        num = (1 + N/2) * (j-m) * (j + m + 1)
-        den = 2 * j * (j+1)
-        pump = yP * (float(num)/den)
-        collective_pump = yCP * (j-m) * (j+m+1)
+        num = (1 + N / 2) * (j - m) * (j + m + 1)
+        den = 2 * j * (j + 1)
+        pump = yP * (float(num) / den)
+        collective_pump = yCP * (j - m) * (j + m + 1)
         t8 = pump + collective_pump
         return t8
 
@@ -1636,7 +1947,7 @@ class Pim(object):
         """
         yP = self.pumping
         N = float(self.N)
-        num = (j+m+1) * (j+m+2) * (N/2 - j)
-        den = 2 * (j+1) * (2*j + 1)
-        t9 = yP * (float(num)/den)
+        num = (j + m + 1) * (j + m + 2) * (N / 2 - j)
+        den = 2 * (j + 1) * (2 * j + 1)
+        t9 = yP * (float(num) / den)
         return t9
