@@ -133,7 +133,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
     if isinstance(H, (types.FunctionType, types.BuiltinFunctionType,
                       functools.partial)):
         H0 = H(0.0, args)
-        if unitary_mode =='batch':
+        if unitary_mode == 'batch':
             # batch don't work with function Hamiltonian
             unitary_mode = 'single'
     elif isinstance(H, list):
@@ -160,12 +160,9 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
             if unitary_mode == 'single':
                 output = sesolve(H, qeye(dims[0]), tlist, [], args, options,
                                  _safe_mode=False)
-                if len(tlist) == 2:
-                    return output.states[-1]
-                else:
-                    return output.states
+                return output.states[-1] if len(tlist) == 2 else output.states
 
-            elif unitary_mode =='batch':
+            elif unitary_mode == 'batch':
                 u = np.zeros(len(tlist), dtype=object)
                 _rows = np.array([(N+1)*m for m in range(N)])
                 _cols = np.zeros_like(_rows)
@@ -188,10 +185,8 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                     u[k] = sp_reshape(output.states[k].data, (N, N))
                     unit_row_norm(u[k].data, u[k].indptr, u[k].shape[0])
                     u[k] = u[k].T.tocsr()
-
             else:
-                raise Exception('Invalid unitary mode.')
-
+                raise ValueError('Invalid unitary mode.')
 
     elif len(c_op_list) == 0 and H0.issuper:
         # calculate the propagator for the vector representation of the
@@ -204,7 +199,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
         u = np.zeros([N, N, len(tlist)], dtype=complex)
 
         if parallel:
-            output = parallel_map(_parallel_mesolve,range(N * N),
+            output = parallel_map(_parallel_mesolve, range(N * N),
                                   task_args=(
                                       sqrt_N, H, tlist, c_op_list, args,
                                       options),
@@ -213,14 +208,11 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                 for k, t in enumerate(tlist):
                     u[:, n, k] = mat2vec(output[n].states[k].full()).T
         else:
-            rho0 = qeye(N,N)
+            rho0 = qeye(N, N)
             rho0.dims = [[sqrt_N, sqrt_N], [sqrt_N, sqrt_N]]
             output = mesolve(H, psi0, tlist, [], args, options,
                              _safe_mode=False)
-            if len(tlist) == 2:
-                return output.states[-1]
-            else:
-                return output.states
+            return output.states[-1] if len(tlist) == 2 else output.states
 
     else:
         # calculate the propagator for the vector representation of the
@@ -245,7 +237,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                 progress_bar.update(n)
                 col_idx, row_idx = np.unravel_index(n, (N, N))
                 rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
-                                          shape=(N,N), dtype=complex))
+                                          shape=(N, N), dtype=complex))
                 output = mesolve(H, rho0, tlist, c_op_list, [], args, options,
                                  _safe_mode=False)
                 for k, t in enumerate(tlist):
@@ -253,17 +245,15 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
             progress_bar.finished()
 
     if len(tlist) == 2:
-        if unitary_mode == 'batch':
-            return Qobj(u[-1], dims=dims)
-        else:
-            return Qobj(u[:, :, 1], dims=dims)
+        data = u[-1] if unitary_mode == 'batch' else u[:, :, 1]
+        return Qobj(data, dims=dims)
+
+    out = np.empty((len(tlist),), dtype=object)
+    if unitary_mode == 'batch':
+        out[:] = [Qobj(u[k], dims=dims) for k in range(len(tlist))]
     else:
-        if unitary_mode == 'batch':
-            return np.array([Qobj(u[k], dims=dims)
-                             for k in range(len(tlist))], dtype=object)
-        else:
-            return np.array([Qobj(u[:, :, k], dims=dims)
-                             for k in range(len(tlist))], dtype=object)
+        out[:] = [Qobj(u[:, :, k], dims=dims) for k in range(len(tlist))]
+    return out
 
 
 def _get_min_and_index(lst):
@@ -311,10 +301,11 @@ def _parallel_sesolve(n, N, H, tlist, args, options):
     output = sesolve(H, psi0, tlist, [], args, options, _safe_mode=False)
     return output
 
+
 def _parallel_mesolve(n, N, H, tlist, c_op_list, args, options):
     col_idx, row_idx = np.unravel_index(n, (N, N))
     rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
-                              shape=(N,N), dtype=complex))
+                              shape=(N, N), dtype=complex))
     output = mesolve(H, rho0, tlist, c_op_list, [], args, options,
                      _safe_mode=False)
     return output
