@@ -40,10 +40,11 @@ __all__ = ['Qobj', 'qobj_list_evaluate', 'ptrace', 'dag', 'isequal',
 
 import warnings
 import types
+import numbers
 
 try:
     import builtins
-except:
+except ImportError:
     import __builtin__ as builtins
 
 # import math functions from numpy.math: required for td string evaluation
@@ -60,11 +61,16 @@ from qutip import __version__
 from qutip.fastsparse import fast_csr_matrix, fast_identity
 from qutip.cy.ptrace import _ptrace
 from qutip.permute import _permute
-from qutip.sparse import (sp_eigs, sp_expm, sp_fro_norm, sp_max_norm,
-                          sp_one_norm, sp_L2_norm)
-from qutip.dimensions import type_from_dims, enumerate_flat, collapse_dims_super
-from qutip.cy.spmath import (zcsr_transpose, zcsr_adjoint, zcsr_isherm,
-                            zcsr_trace, zcsr_proj, zcsr_inner)
+from qutip.sparse import (
+    sp_eigs, sp_expm, sp_fro_norm, sp_max_norm, sp_one_norm, sp_L2_norm,
+)
+from qutip.dimensions import (
+    type_from_dims, enumerate_flat, collapse_dims_super,
+)
+from qutip.cy.spmath import (
+    zcsr_transpose, zcsr_adjoint, zcsr_isherm, zcsr_trace, zcsr_proj,
+    zcsr_inner,
+)
 from qutip.cy.spmatfuncs import zcsr_mat_elem
 from qutip.cy.sparse_utils import cy_tidyup
 import sys
@@ -74,7 +80,7 @@ elif sys.version_info.major < 3:
     from itertools import izip_longest
     zip_longest = izip_longest
 
-#OPENMP stuff
+# OPENMP stuff
 from qutip.cy.openmp.utilities import use_openmp
 if settings.has_openmp:
     from qutip.cy.openmp.omp_sparse_utils import omp_tidyup
@@ -176,6 +182,8 @@ class Qobj(object):
     groundstate(sparse=False, tol=0, maxiter=100000)
         Returns eigenvalue and eigenket for the groundstate of a quantum
         object.
+    inv()
+        Return a Qobj corresponding to the matrix inverse of the operator.
     matrix_element(bra, ket)
         Returns the matrix element of operator between `bra` and `ket` vectors.
     norm(norm='tr', sparse=False, tol=0, maxiter=100000)
@@ -207,8 +215,11 @@ class Qobj(object):
 
     """
     __array_priority__ = 100  # sets Qobj priority above numpy arrays
+    # Disable ufuncs from acting directly on Qobj. This is necessary because we
+    # define __array__.
+    __array_ufunc__ = None
 
-    def __init__(self, inpt=None, dims=[[], []], shape=[],
+    def __init__(self, inpt=None, dims=None, shape=None,
                  type=None, isherm=None, copy=True,
                  fast=False, superrep=None, isunitary=None):
         """
@@ -236,11 +247,12 @@ class Qobj(object):
         if isinstance(inpt, Qobj):
             # if input is already Qobj then return identical copy
 
-            self._data = fast_csr_matrix((inpt.data.data, inpt.data.indices,
-                                          inpt.data.indptr),
-                                          shape=inpt.shape, copy=copy)
+            self._data = fast_csr_matrix(
+                (inpt.data.data, inpt.data.indices, inpt.data.indptr),
+                shape=inpt.shape, copy=copy,
+            )
 
-            if not np.any(dims):
+            if dims is None:
                 # Dimensions of quantum object used for keeping track of tensor
                 # components
                 self.dims = inpt.dims
@@ -253,11 +265,11 @@ class Qobj(object):
         elif inpt is None:
             # initialize an empty Qobj with correct dimensions and shape
 
-            if any(dims):
+            if dims is not None:
                 N, M = np.prod(dims[0]), np.prod(dims[1])
                 self.dims = dims
 
-            elif shape:
+            elif shape is not None:
                 N, M = shape
                 self.dims = [[N], [M]]
 
@@ -275,9 +287,11 @@ class Qobj(object):
                 data = data.transpose()
 
             _tmp = sp.csr_matrix(data, dtype=complex)
-            self._data = fast_csr_matrix((_tmp.data, _tmp.indices, _tmp.indptr),
-                                         shape=_tmp.shape)
-            if not np.any(dims):
+            self._data = fast_csr_matrix(
+                (_tmp.data, _tmp.indices, _tmp.indptr),
+                shape=_tmp.shape,
+            )
+            if dims is None:
                 self.dims = [[int(data.shape[0])], [int(data.shape[1])]]
             else:
                 self.dims = dims
@@ -290,14 +304,17 @@ class Qobj(object):
             do_copy = copy
             if not isinstance(inpt, fast_csr_matrix):
                 _tmp = sp.csr_matrix(inpt, dtype=complex, copy=do_copy)
-                _tmp.sort_indices() #Make sure indices are sorted.
+                _tmp.sort_indices()  # Make sure indices are sorted.
                 do_copy = 0
             else:
                 _tmp = inpt
-            self._data = fast_csr_matrix((_tmp.data, _tmp.indices, _tmp.indptr),
-                                        shape=_tmp.shape, copy=do_copy)
+            self._data = fast_csr_matrix(
+                (_tmp.data, _tmp.indices, _tmp.indptr),
+                shape=_tmp.shape,
+                copy=do_copy,
+            )
 
-            if not np.any(dims):
+            if dims is None:
                 self.dims = [[int(inpt.shape[0])], [int(inpt.shape[1])]]
             else:
                 self.dims = dims
@@ -306,9 +323,11 @@ class Qobj(object):
                                np.integer, np.floating, np.complexfloating)):
             # if input is int, float, or complex then convert to array
             _tmp = sp.csr_matrix([[inpt]], dtype=complex)
-            self._data = fast_csr_matrix((_tmp.data, _tmp.indices, _tmp.indptr),
-                                        shape=_tmp.shape)
-            if not np.any(dims):
+            self._data = fast_csr_matrix(
+                (_tmp.data, _tmp.indices, _tmp.indptr),
+                shape=_tmp.shape,
+            )
+            if dims is None:
                 self.dims = [[1], [1]]
             else:
                 self.dims = dims
@@ -318,13 +337,16 @@ class Qobj(object):
                           builtins.type(inpt))
             inpt = np.array([[0]])
             _tmp = sp.csr_matrix(inpt, dtype=complex, copy=copy)
-            self._data = fast_csr_matrix((_tmp.data, _tmp.indices, _tmp.indptr),
-                                        shape = _tmp.shape)
+            self._data = fast_csr_matrix(
+                (_tmp.data, _tmp.indices, _tmp.indptr),
+                shape=_tmp.shape,
+            )
             self.dims = [[int(inpt.shape[0])], [int(inpt.shape[1])]]
 
         if type == 'super':
-            # Type is not super, i.e. dims not explicitly passed, but oper shape
-            if dims== [[], []] and self.shape[0] == self.shape[1]:
+            # Type is not super, i.e. dims not explicitly passed, but oper-like
+            # shape.
+            if dims is None and self.shape[0] == self.shape[1]:
                 sub_shape = np.sqrt(self.shape[0])
                 # check if root of shape is int
                 if (sub_shape % 1) != 0:
@@ -333,7 +355,6 @@ class Qobj(object):
                     sub_shape = int(sub_shape)
                     self.dims = [[[sub_shape], [sub_shape]]]*2
 
-
         if superrep:
             self.superrep = superrep
         else:
@@ -349,7 +370,8 @@ class Qobj(object):
 
     def get_data(self):
         return self._data
-    #Here we perfrom a check of the csr matrix type during setting of Q.data
+
+    # Here we perfrom a check of the csr matrix type during setting of Q.data
     def set_data(self, data):
         if not isinstance(data, fast_csr_matrix):
             raise TypeError('Qobj data must be in fast_csr format.')
@@ -391,15 +413,15 @@ class Qobj(object):
 
             out.dims = self.dims
 
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
 
             if isinstance(dat, (int, float)):
                 out._isherm = self._isherm
             else:
                 # We use _isherm here to prevent recalculating on self and
                 # other, relying on that bool(None) == False.
-                out._isherm = (True if self._isherm and other._isherm
-                                    else out.isherm)
+                out._isherm = (self._isherm and other._isherm) or out.isherm
 
             out.superrep = self.superrep
 
@@ -419,7 +441,8 @@ class Qobj(object):
                 out.data.data = out.data.data + dat
             out.dims = other.dims
 
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
 
             if isinstance(dat, complex):
                 out._isherm = out.isherm
@@ -440,7 +463,8 @@ class Qobj(object):
             out = Qobj()
             out.data = self.data + other.data
             out.dims = self.dims
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
 
             if self.type in ['ket', 'bra', 'operator-ket', 'operator-bra']:
                 out._isherm = False
@@ -491,7 +515,8 @@ class Qobj(object):
                 out.data = self.data * other.data
                 dims = [self.dims[0], other.dims[1]]
                 out.dims = dims
-                if settings.auto_tidyup: out.tidyup()
+                if settings.auto_tidyup:
+                    out.tidyup()
                 if (settings.auto_tidyup_dims
                         and not isinstance(dims[0][0], list)
                         and not isinstance(dims[1][0], list)):
@@ -502,9 +527,9 @@ class Qobj(object):
                     # to have uneven length (non-square Qobjs).
                     # We use None as padding so that it doesn't match anything,
                     # and will never cause a partial trace on the other side.
-                    mask = [l == r == 1 for l, r in zip_longest(dims[0],
-                                                                dims[1],
-                                                                fillvalue=None)]
+                    mask = [l == r == 1
+                            for l, r in zip_longest(dims[0], dims[1],
+                                                    fillvalue=None)]
                     # To ensure that there are still any dimensions left, we
                     # use max() to add a dimensions list of [1] if all matching
                     # dims are traced out of that side.
@@ -542,32 +567,40 @@ class Qobj(object):
                 out.superrep = self.superrep
                 return out.tidyup() if settings.auto_tidyup else out
 
+            elif self.shape[1] == 1 and other.shape[0] == 1:
+                out = Qobj()
+                out.data = self.data * other.data
+                out.dims = [self.dims[0], other.dims[1]]
+                return out.tidyup() if settings.auto_tidyup else out
+
             else:
                 raise TypeError("Incompatible Qobj shapes")
 
         elif isinstance(other, np.ndarray):
-            if other.dtype=='object':
-                return np.array([self * item for item in other],
-                                dtype=object)
+            if other.dtype == 'object':
+                out = np.empty(other.shape, dtype=object)
+                for i, item in enumerate(other.flat):
+                    out.flat[i] = self * item
+                return out
             else:
                 return self.data * other
 
-
         elif isinstance(other, list):
             # if other is a list, do element-wise multiplication
-            return np.array([self * item for item in other],
-                            dtype=object)
+            out = np.empty(len(other), dtype=object)
+            out[:] = [self * item for item in other]
+            return out
 
         elif isinstance(other, eseries):
             return other.__rmul__(self)
 
-        elif isinstance(other, (int, float, complex,
-                                np.integer, np.floating, np.complexfloating)):
+        elif isinstance(other, numbers.Number):
             out = Qobj()
             out.data = self.data * other
             out.dims = self.dims
             out.superrep = self.superrep
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
             if isinstance(other, complex):
                 out._isherm = out.isherm
             else:
@@ -583,27 +616,30 @@ class Qobj(object):
         MULTIPLICATION with Qobj on RIGHT [ ex. 4*Qobj ]
         """
         if isinstance(other, np.ndarray):
-            if other.dtype=='object':
-                return np.array([item * self for item in other],
-                                            dtype=object)
+            if other.dtype == 'object':
+                out = np.empty(other.shape, dtype=object)
+                for i, item in enumerate(other.flat):
+                    out.flat[i] = item * self
+                return out
             else:
                 return other * self.data
 
         elif isinstance(other, list):
             # if other is a list, do element-wise multiplication
-            return np.array([item * self for item in other],
-                            dtype=object)
+            out = np.empty(len(other), dtype=object)
+            out[:] = [item * self for item in other]
+            return out
 
         elif isinstance(other, eseries):
             return other.__mul__(self)
 
-        elif isinstance(other, (int, float, complex,
-                              np.integer, np.floating, np.complexfloating)):
+        elif isinstance(other, numbers.Number):
             out = Qobj()
             out.data = other * self.data
             out.dims = self.dims
             out.superrep = self.superrep
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
             if isinstance(other, complex):
                 out._isherm = out.isherm
             else:
@@ -630,7 +666,8 @@ class Qobj(object):
             out = Qobj()
             out.data = self.data / other
             out.dims = self.dims
-            if settings.auto_tidyup: out.tidyup()
+            if settings.auto_tidyup:
+                out.tidyup()
             if isinstance(other, complex):
                 out._isherm = out.isherm
             else:
@@ -651,7 +688,8 @@ class Qobj(object):
         out.data = -self.data
         out.dims = self.dims
         out.superrep = self.superrep
-        if settings.auto_tidyup: out.tidyup()
+        if settings.auto_tidyup:
+            out.tidyup()
         out._isherm = self._isherm
         out._isunitary = self._isunitary
         return out
@@ -697,12 +735,11 @@ class Qobj(object):
 
         try:
             data = self.data ** n
-            out = Qobj(data, dims=self.dims)
-            out.superrep = self.superrep
-            return out.tidyup() if settings.auto_tidyup else out
-
-        except:
+        except (TypeError, ValueError):
             raise ValueError('Invalid choice of exponent.')
+        out = Qobj(data, dims=self.dims)
+        out.superrep = self.superrep
+        return out.tidyup() if settings.auto_tidyup else out
 
     def __abs__(self):
         return abs(self.data)
@@ -931,7 +968,6 @@ class Qobj(object):
         J_dual.superrep = 'choi'
         return J_dual
 
-
     def conj(self):
         """Conjugate operator of quantum object.
         """
@@ -1015,16 +1051,15 @@ class Qobj(object):
             Projection operator.
         """
         if self.isket:
-            _out = zcsr_proj(self.data,1)
-            _dims = [self.dims[0],self.dims[0]]
+            _out = zcsr_proj(self.data, 1)
+            _dims = [self.dims[0], self.dims[0]]
         elif self.isbra:
-            _out = zcsr_proj(self.data,0)
-            _dims = [self.dims[1],self.dims[1]]
+            _out = zcsr_proj(self.data, 0)
+            _dims = [self.dims[1], self.dims[1]]
         else:
             raise TypeError('Projector can only be formed from a bra or ket.')
 
-        return Qobj(_out,dims=_dims)
-
+        return Qobj(_out, dims=_dims)
 
     def tr(self):
         """Trace of a quantum object.
@@ -1199,7 +1234,6 @@ class Qobj(object):
         else:
             raise TypeError('Invalid operand for matrix square root')
 
-
     def cosm(self):
         """Cosine of a quantum operator.
 
@@ -1221,11 +1255,10 @@ class Qobj(object):
 
         """
         if self.dims[0][0] == self.dims[1][0]:
-             return 0.5 * ((1j * self).expm() + (-1j * self).expm())
+            return 0.5 * ((1j * self).expm() + (-1j * self).expm())
         else:
             raise TypeError('Invalid operand for matrix square root')
 
-
     def sinm(self):
         """Sine of a quantum operator.
 
@@ -1247,15 +1280,36 @@ class Qobj(object):
 
         """
         if self.dims[0][0] == self.dims[1][0]:
-             return -0.5j * ((1j * self).expm() - (-1j * self).expm())
+            return -0.5j * ((1j * self).expm() - (-1j * self).expm())
         else:
             raise TypeError('Invalid operand for matrix square root')
 
+    def inv(self, sparse=False):
+        """Matrix inverse of a quantum operator
 
+        Operator must be square.
+
+        Returns
+        -------
+        oper : :class:`qutip.Qobj`
+            Matrix inverse of operator.
+
+        Raises
+        ------
+        TypeError
+            Quantum object is not square.
+        """
+        if self.shape[0] != self.shape[1]:
+            raise TypeError('Invalid operand for matrix inverse')
+        if sparse:
+            inv_mat = sp.linalg.inv(sp.csc_matrix(self.data))
+        else:
+            inv_mat = np.linalg.inv(self.full())
+        return Qobj(inv_mat, dims=self.dims[::-1])
 
     def unit(self, inplace=False,
-            norm=None, sparse=False,
-            tol=0, maxiter=100000):
+             norm=None, sparse=False,
+             tol=0, maxiter=100000):
         """Operator or state normalized to unity.
 
         Uses norm from Qobj.norm().
@@ -1282,7 +1336,7 @@ class Qobj(object):
         """
         if inplace:
             nrm = self.norm(norm=norm, sparse=sparse,
-                           tol=tol, maxiter=maxiter)
+                            tol=tol, maxiter=maxiter)
 
             self.data /= nrm
         elif not inplace:
@@ -1295,7 +1349,7 @@ class Qobj(object):
         else:
             raise Exception('inplace kwarg must be bool.')
 
-    def ptrace(self, sel):
+    def ptrace(self, sel, sparse=None):
         """Partial trace of the quantum object.
 
         Parameters
@@ -1315,9 +1369,17 @@ class Qobj(object):
         that has been deprecated.
 
         """
-        q = Qobj()
-        q.data, q.dims, _ = _ptrace(self, sel)
-        return q.tidyup() if settings.auto_tidyup else q
+        if sparse is None:
+            if self.isket:
+                sparse = False
+            elif (self.data.nnz / (self.shape[0] * self.shape[1])) >= 0.1:
+                sparse = False
+        if sparse:
+            q = Qobj()
+            q.data, q.dims, _ = _ptrace(self, sel)
+            return q.tidyup() if settings.auto_tidyup else q
+        else:
+            return _ptrace_dense(self, sel)
 
     def permute(self, order):
         """Permutes a composite quantum object.
@@ -1336,7 +1398,7 @@ class Qobj(object):
         q = Qobj()
         q.data, q.dims = _permute(self, order)
         q.data.sort_indices()
-        return q.tidyup() if settings.auto_tidyup else q
+        return q
 
     def tidyup(self, atol=settings.auto_tidyup_atol):
         """Removes small elements from the quantum object.
@@ -1354,14 +1416,14 @@ class Qobj(object):
 
         """
         if self.data.nnz:
-            #This does the tidyup and returns True if
-            #The sparse data needs to be shortened
+            # This does the tidyup and returns True if
+            # The sparse data needs to be shortened
             if use_openmp() and self.data.nnz > 500:
-                if omp_tidyup(self.data.data,atol,self.data.nnz,
-                            settings.num_cpus):
-                            self.data.eliminate_zeros()
+                if omp_tidyup(self.data.data, atol, self.data.nnz,
+                              settings.num_cpus):
+                    self.data.eliminate_zeros()
             else:
-                if cy_tidyup(self.data.data,atol,self.data.nnz):
+                if cy_tidyup(self.data.data, atol, self.data.nnz):
                     self.data.eliminate_zeros()
             return self
         else:
@@ -1401,10 +1463,10 @@ class Qobj(object):
                     'Invalid size of ket list for basis transformation')
             if sparse:
                 S = sp.hstack([psi.data for psi in inpt],
-                            format='csr', dtype=complex).conj().T
+                              format='csr', dtype=complex).conj().T
             else:
                 S = np.hstack([psi.full() for psi in inpt],
-                               dtype=complex).conj().T
+                              dtype=complex).conj().T
         elif isinstance(inpt, Qobj) and inpt.isoper:
             S = inpt.data
         elif isinstance(inpt, np.ndarray):
@@ -1413,11 +1475,10 @@ class Qobj(object):
         else:
             raise TypeError('Invalid operand for basis transformation')
 
-
         # transform data
         if inverse:
             if self.isket:
-                  data = (S.conj().T) * self.data
+                data = (S.conj().T) * self.data
             elif self.isbra:
                 data = self.data.dot(S)
             else:
@@ -1445,8 +1506,6 @@ class Qobj(object):
         else:
             return out
 
-
-
     def trunc_neg(self, method="clip"):
         """Truncates negative eigenvalues and renormalizes.
 
@@ -1471,7 +1530,7 @@ class Qobj(object):
         """
         if not self.isherm:
             raise ValueError("Must be a Hermitian operator to remove negative "
-                            "eigenvalues.")
+                             "eigenvalues.")
 
         if method not in ('clip', 'sgs'):
             raise ValueError("Method {} not recognized.".format(method))
@@ -1509,7 +1568,6 @@ class Qobj(object):
             ], Qobj(np.zeros(self.shape), dims=self.dims)
         ).unit()
 
-
     def matrix_element(self, bra, ket):
         """Calculates a matrix element.
 
@@ -1540,10 +1598,10 @@ class Qobj(object):
 
         else:
             if bra.isbra and ket.isket:
-                return zcsr_mat_elem(self.data,bra.data,ket.data,1)
+                return zcsr_mat_elem(self.data, bra.data, ket.data, 1)
 
             elif bra.isket and ket.isket:
-                return zcsr_mat_elem(self.data,bra.data,ket.data,0)
+                return zcsr_mat_elem(self.data, bra.data, ket.data, 0)
             else:
                 err = "Can only calculate matrix elements for bra"
                 err += " and ket vectors."
@@ -1586,8 +1644,8 @@ class Qobj(object):
                 if other.isket:
                     return zcsr_inner(self.data, other.data, 1)
                 elif other.isbra:
-                    #Since we deal mainly with ket vectors, the bra-bra combo
-                    #is not common, and not optimized.
+                    # Since we deal mainly with ket vectors, the bra-bra combo
+                    # is not common, and not optimized.
                     return zcsr_inner(self.data, other.dag().data, 1)
                 elif other.isoper:
                     return (qutip.states.ket2dm(self).dag() * other).tr()
@@ -1614,11 +1672,8 @@ class Qobj(object):
                 else:
                     err = "Can only calculate overlap for state vector Qobjs"
                     raise TypeError(err)
-
-
         raise TypeError("Can only calculate overlap for state vector Qobjs")
 
-
     def eigenstates(self, sparse=False, sort='low',
                     eigvals=0, tol=0, maxiter=100000):
         """Eigenstates and eigenenergies.
@@ -1663,12 +1718,11 @@ class Qobj(object):
                                sort=sort, eigvals=eigvals, tol=tol,
                                maxiter=maxiter)
         new_dims = [self.dims[0], [1] * len(self.dims[0])]
-        ekets = np.array([Qobj(vec, dims=new_dims) for vec in evecs],
-                         dtype=object)
+        ekets = np.empty((len(evecs),), dtype=object)
+        ekets[:] = [Qobj(vec, dims=new_dims) for vec in evecs]
         norms = np.array([ket.norm() for ket in ekets])
         return evals, ekets / norms
 
-
     def eigenenergies(self, sparse=False, sort='low',
                       eigvals=0, tol=0, maxiter=100000):
         """Eigenenergies of a quantum object.
@@ -1741,10 +1795,10 @@ class Qobj(object):
         grndval, grndvec = sp_eigs(self.data, self.isherm, sparse=sparse,
                                    eigvals=evals, tol=tol, maxiter=maxiter)
         if safe:
-            if tol == 0: tol = 1e-15
+            tol = 1e-15 if tol == 0 else tol
             if (grndval[1]-grndval[0]) <= 10*tol:
                 print("WARNING: Ground state may be degenerate. "
-                        "Use Q.eigenstates()")
+                      "Use Q.eigenstates()")
         new_dims = [self.dims[0], [1] * len(self.dims[0])]
         grndvec = Qobj(grndvec[0], dims=new_dims)
         grndvec = grndvec / grndvec.norm()
@@ -1841,7 +1895,8 @@ class Qobj(object):
         ----------
         B : :class:`qutip.Qobj` or None
             If B is not None, the diamond distance d(A, B) = dnorm(A - B)
-            between this operator and B is returned instead of the diamond norm.
+            between this operator and B is returned instead of the diamond
+            norm.
 
         Returns
         -------
@@ -1852,7 +1907,6 @@ class Qobj(object):
         """
         return mts.dnorm(self, B)
 
-
     @property
     def ishp(self):
         # FIXME: this needs to be cached in the same ways as isherm.
@@ -1860,7 +1914,7 @@ class Qobj(object):
             try:
                 J = sr.to_choi(self)
                 return J.isherm
-            except:
+            except TypeError:
                 return False
         else:
             return False
@@ -1887,7 +1941,7 @@ class Qobj(object):
                     return False
                 eigs = J.eigenenergies()
                 return all(eigs >= -settings.atol)
-            except:
+            except TypeError:
                 return False
         else:
             return False
@@ -1907,8 +1961,10 @@ class Qobj(object):
                     qobj = sr.to_choi(self)
 
                 # Possibly collapse dims.
-                if any([len(index) > 1 for super_index in qobj.dims
-                                            for index in super_index]):
+                if any(
+                    len(index) > 1
+                    for super_index in qobj.dims for index in super_index
+                ):
                     qobj = Qobj(qobj, dims=collapse_dims_super(qobj.dims))
                 else:
                     qobj = qobj
@@ -1918,7 +1974,7 @@ class Qobj(object):
                 tr_oper = qobj.ptrace([0])
                 ident = ops.identity(tr_oper.shape[0])
                 return isequal(tr_oper, ident)
-            except:
+            except TypeError:
                 return False
         else:
             return False
@@ -1935,7 +1991,6 @@ class Qobj(object):
 
     @property
     def isherm(self):
-
         if self._isherm is not None:
             # used previously computed value
             return self._isherm
@@ -1954,15 +2009,17 @@ class Qobj(object):
         """
         if self.isoper:
             eye_data = fast_identity(self.shape[0])
-            return not (np.any(np.abs((self.data*self.dag().data
-                                       - eye_data).data)
-                                > settings.atol)
-                        or
-                        np.any(np.abs((self.dag().data*self.data
-                                       - eye_data).data) >
-                              settings.atol)
-                        )
-
+            return not (
+                np.any(
+                    np.abs((self.data*self.dag().data - eye_data).data)
+                    > settings.atol
+                )
+                or
+                np.any(
+                    np.abs((self.dag().data*self.data - eye_data).data)
+                    > settings.atol
+                )
+            )
         else:
             return False
 
@@ -2102,7 +2159,6 @@ def qobj_list_evaluate(qobj_list, t, args):
 # A collection of tests used to determine the type of quantum objects, and some
 # functions for increased compatibility with quantum optics toolbox.
 #
-
 def dag(A):
     """Adjont operator (dagger) of a quantum object.
 
@@ -2156,6 +2212,36 @@ def ptrace(Q, sel):
     return Q.ptrace(sel)
 
 
+def _ptrace_dense(Q, sel):
+    rd = np.asarray(Q.dims[0], dtype=np.int32).ravel()
+    nd = len(rd)
+    if isinstance(sel, int):
+        sel = np.array([sel])
+    else:
+        sel = np.asarray(sel)
+    sel = list(np.sort(sel))
+    dkeep = (rd[sel]).tolist()
+    qtrace = list(set(np.arange(nd)) - set(sel))
+    dtrace = (rd[qtrace]).tolist()
+    rd = list(rd)
+    if isket(Q):
+        vmat = (Q.full()
+                .reshape(rd)
+                .transpose(sel + qtrace)
+                .reshape([np.prod(dkeep), np.prod(dtrace)]))
+        rhomat = vmat.dot(vmat.conj().T)
+    else:
+        rhomat = np.trace(Q.full()
+                          .reshape(rd + rd)
+                          .transpose(qtrace + [nd + q for q in qtrace] +
+                                     sel + [nd + q for q in sel])
+                          .reshape([np.prod(dtrace),
+                                    np.prod(dtrace),
+                                    np.prod(dkeep),
+                                    np.prod(dkeep)]))
+    return Qobj(rhomat, dims=[dkeep, dkeep])
+
+
 def dims(inpt):
     """Returns the dims attribute of a quantum object.
 
