@@ -58,7 +58,7 @@ from qutip.superoperator import liouvillian, vec2mat
 from qutip.sparse import (sp_permute, sp_bandwidth, sp_reshape,
                           sp_profile)
 from qutip.cy.spmath import zcsr_kron
-from qutip.graph import reverse_cuthill_mckee, weighted_bipartite_matching
+from qutip.graph import weighted_bipartite_matching
 from qutip import (mat2vec, tensor, identity, operator_to_vector)
 import qutip.settings as settings
 from qutip.utilities import _version2int
@@ -86,29 +86,17 @@ def _empty_info_dict():
 
 def _default_steadystate_args():
     def_args = {'sparse': True, 'use_rcm': False,
-                'use_wbm': False, 'weight': None, 'use_precond': False,
+                'use_wbm': False, 'use_precond': False,
                 'all_states': False, 'M': None, 'x0': None, 'drop_tol': 1e-4,
                 'fill_factor': 100, 'diag_pivot_thresh': None, 'maxiter': 1000,
-                'tol': 1e-12, 'matol': 1e-15, 'mtol': None,
                 'permc_spec': 'COLAMD', 'ILU_MILU': 'smilu_2',
-                'restart': 20, 'return_info': False,
-                'info': _empty_info_dict(),
-                'verbose': False, 'solver': 'scipy'}
-
-    return def_args
-
-
-def _mkl_steadystate_args():
-    def_args = {'max_iter_refine': 10,
+                'restart': 20,
+                'max_iter_refine': 10,
                 'scaling_vectors': True,
                 'weighted_matching': True,
                 'return_info': False, 'info': _empty_info_dict(),
-                'verbose': False, 'solver': 'mkl',
-                'use_rcm': False,
-                'use_wbm': False, 'weight': None,
-                'tol': 1e-12, 'matol': 1e-15, 'mtol': None,
-                'maxiter': 1000}
-
+                'verbose': False, 'solver': 'scipy', 'weight': None,
+                'tol': 1e-12, 'matol': 1e-15, 'mtol': None}
     return def_args
 
 
@@ -255,13 +243,10 @@ def steadystate(A, c_op_list=[], method='direct', solver=None, **kwargs):
     elif solver not in ['scipy', 'mkl']:
         raise Exception('Invalid solver kwarg.')
 
-    if solver == 'scipy':
-        ss_args = _default_steadystate_args()
-    elif solver == 'mkl':
-        ss_args = _mkl_steadystate_args()
-    else:
-        raise Exception('Invalid solver keyword argument.')
+    ss_args = _default_steadystate_args()
     ss_args['method'] = method
+    if solver is not None:
+        ss_args['solver'] = solver
     ss_args['info']['solver'] = ss_args['solver']
     ss_args['info']['method'] = ss_args['method']
 
@@ -368,7 +353,7 @@ def _steadystate_LU_liouvillian(L, ss_args, has_mkl=0):
         if settings.debug:
             logger.debug('Calculating Reverse Cuthill-Mckee ordering...')
         _rcm_start = time.time()
-        perm2 = reverse_cuthill_mckee(L)
+        perm2 = sp.csgraph.reverse_cuthill_mckee(L)
         _rcm_end = time.time()
         rev_perm = np.argsort(perm2)
         L = sp_permute(L, perm2, perm2, form)
@@ -523,7 +508,7 @@ def _steadystate_eigen(L, ss_args):
         if settings.debug:
             old_band = sp_bandwidth(L)[0]
             logger.debug('Original bandwidth: %i' % old_band)
-        perm = reverse_cuthill_mckee(L)
+        perm = sp.csgraph.reverse_cuthill_mckee(L)
         rev_perm = np.argsort(perm)
         L = sp_permute(L, perm, perm, 'csc')
         if settings.debug:
@@ -640,13 +625,14 @@ def _steadystate_iterative(L, ss_args):
     _iter_start = time.time()
     # FIXME: These atol keyword except checks can be removed once scipy 1.1
     # is a minimum requirement
+    extra = {"callback_type": 'legacy'} if scipy.__version__ >= "1.4" else {}
     if ss_args['method'] == 'iterative-gmres':
         try:
             v, check = gmres(L, b, tol=ss_args['tol'], atol=ss_args['matol'],
                              M=ss_args['M'], x0=ss_args['x0'],
                              restart=ss_args['restart'],
                              maxiter=ss_args['maxiter'],
-                             callback=_iter_count)
+                             callback=_iter_count, **extra)
         except TypeError as e:
             if "unexpected keyword argument 'atol'" in str(e):
                 v, check = gmres(L, b, tol=ss_args['tol'],
@@ -794,7 +780,7 @@ def _steadystate_power_liouvillian(L, ss_args, has_mkl=0):
             logger.debug('Calculating Reverse Cuthill-Mckee ordering...')
         ss_args['info']['perm'].append('rcm')
         _rcm_start = time.time()
-        perm2 = reverse_cuthill_mckee(L)
+        perm2 = sp.csgraph.reverse_cuthill_mckee(L)
         _rcm_end = time.time()
         ss_args['info']['rcm_time'] = _rcm_end-_rcm_start
         rev_perm = np.argsort(perm2)
@@ -1161,7 +1147,7 @@ def _pseudo_inverse_sparse(L, rhoss, w=None, **pseudo_args):
             L = 1.0j*(1e-15)*spre(tr_op) + L
 
     if pseudo_args['use_rcm']:
-        perm = reverse_cuthill_mckee(L.data)
+        perm = sp.csgraph.reverse_cuthill_mckee(L.data)
         A = sp_permute(L.data, perm, perm)
         Q = sp_permute(Q, perm, perm)
     else:
