@@ -54,7 +54,7 @@ from numpy.testing import (
 from scipy.optimize import check_grad
 
 from qutip import Qobj, identity, sigmax, sigmay, sigmaz, tensor
-from qutip.qip import hadamard_transform
+from qutip.qip.operations.gates import hadamard_transform
 from qutip.qip.algorithms import qft
 import qutip.control.optimconfig as optimconfig
 import qutip.control.dynamics as dynamics
@@ -342,6 +342,7 @@ class TestPulseOptim:
         control: data dumping
         Dump out processing data, check file counts
         """
+        self.setUp()
         N_EXP_OPTIMDUMP_FILES = 10
         N_EXP_DYNDUMP_FILES = 49
 
@@ -401,6 +402,7 @@ class TestPulseOptim:
             dyn.dump.writeout(f)
         assert_(os.stat(fpath).st_size > 0,
                 msg="Nothing written to dynamics dump file")
+        self.tearDown()
 
     def test_04_unitarity(self):
         """
@@ -430,7 +432,7 @@ class TestPulseOptim:
         assert_(dyn.dump is not None, msg='dynamics dump not created')
 
         # Use the dump to check unitarity of all propagators and evo_ops
-        dyn.unitarity_tol = 1e-14
+        dyn.unitarity_tol = 1e-13
         nu_prop = 0
         nu_fwd_evo = 0
         nu_onto_evo = 0
@@ -1024,4 +1026,3 @@ class TestPulseOptim:
 
 if __name__ == "__main__":
     run_module_suite()
-
