@@ -646,10 +646,10 @@ class TestMESolveSuperInit:
 
         fid = self.fidelitycheck(out1, out2, rho0vec)
         assert_(max(abs(1.0-fid)) < me_error, True)
-    
+
     def test_me_interp1(self):
         "mesolve: interp time-dependent collapse operator #1"
-        
+
         N = 10  # number of basis states to consider
         kappa = 0.2  # coupling to oscillator
         tlist = np.linspace(0, 10, 100)
@@ -663,10 +663,10 @@ class TestMESolveSuperInit:
         actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
         avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < 1e-5)
-        
+
     def test_me_interp2(self):
          "mesolve: interp time-dependent collapse operator #2"
-         
+
          N = 10  # number of basis states to consider
          kappa = 0.2  # coupling to oscillator
          tlist = np.linspace(0, 10, 100)
@@ -681,10 +681,10 @@ class TestMESolveSuperInit:
          actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
          avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
          assert_(avg_diff < 1e-5)
-         
+
     def test_me_interp3(self):
         "mesolve: interp time-dependent collapse operator #3"
-        
+
         N = 10  # number of basis states to consider
         kappa = 0.2  # coupling to oscillator
         tlist = np.linspace(0, 10, 100)
@@ -699,10 +699,10 @@ class TestMESolveSuperInit:
         actual_answer = 9.0 * np.exp(-kappa * (1.0 - np.exp(-tlist)))
         avg_diff = np.mean(abs(actual_answer - expt) / actual_answer)
         assert_(avg_diff < 1e-5)
-        
+
     def test_me_interp4(self):
         "mesolve: interp time-dependent collapse operator #4"
-        
+
         N = 10  # number of basis states to consider
         kappa = 0.2  # coupling to oscillator
         tlist = np.linspace(0, 10, 100)
@@ -736,10 +736,11 @@ class TestMESolverMisc:
         times = np.linspace(0.0, 2.0, 100)
         opts = Options(store_states=False, store_final_state=True)
         rho0 = ket2dm(psi0)
-        result = mesolve(H, rho0, times, [], [a.dag()*a,b.dag()*b,c.dag()*c],options=opts)
+        result = mesolve(H, rho0, times, [], [a.dag()*a, b.dag()*b, c.dag()*c],
+                         options=opts)
         assert_(rho0.dims == result.final_state.dims)
-        
-    
+
+
     def testSEFinalState(self):
         "sesolve: final_state has correct dims"
 
@@ -752,7 +753,8 @@ class TestMESolverMisc:
 
         times = np.linspace(0.0, 2.0, 100)
         opts = Options(store_states=False, store_final_state=True)
-        result = mesolve(H, psi0, times, [], [a.dag()*a,b.dag()*b,c.dag()*c],options=opts)
+        result = mesolve(H, psi0, times, [], [a.dag()*a, b.dag()*b, c.dag()*c],
+                         options=opts)
         assert_(psi0.dims == result.final_state.dims)
 
 
@@ -844,6 +846,29 @@ class TestMESolveStepFuncCoeff:
         assert_allclose(
             fidelity(result.states[-1], sigmax()*rho0), 1, rtol=1.e-7)
 
+    def test_dynamic_args(self):
+        "sesolve: state feedback"
+        tol = 1e-3
+        def f(t, args):
+            return np.sqrt(args["state_vec"][3])
+
+        H = [qeye(2), [destroy(2)+create(2), f]]
+        res = mesolve(H, basis(2,1), tlist=np.linspace(0,10,11),
+                      c_ops=[qeye(2)],
+                      e_ops=[num(2)], args={"state_vec":basis(2,1)})
+        assert_(max(abs(res.expect[0][5:])) < tol,
+            msg="evolution with feedback not proceding as expected")
+
+        def f(t, args):
+            return np.sqrt(args["expect_op_0"])
+
+        H = [qeye(2), [destroy(2)+create(2), f]]
+        res = mesolve(H, basis(2,1), tlist=np.linspace(0,10,11),
+                      c_ops=[qeye(2)],
+                      e_ops=[num(2)], args={"expect_op_0":num(2)})
+        assert_(max(abs(res.expect[0][5:])) < tol,
+            msg="evolution with feedback not proceding as expected")
+
 
 if __name__ == "__main__":
     run_module_suite()
