@@ -32,10 +32,15 @@
 ###############################################################################
 
 import numpy as np
-from numpy.testing import assert_, run_module_suite
-from qutip.qip.gates import gate_sequence_product
+from numpy.testing import assert_, run_module_suite, assert_allclose
+
+from qutip.qip.operations.gates import gate_sequence_product
 from qutip.qip.circuit import QubitCircuit
-from qutip.qip.models.spinchain import LinearSpinChain, CircularSpinChain
+from qutip.qip.device.spinchain import LinearSpinChain, CircularSpinChain
+from qutip.random_objects import rand_ket
+from qutip.metrics import fidelity
+from qutip.operators import sigmaz, sigmax
+from qutip.solver import Options
 
 
 class TestSpinChain:
@@ -154,6 +159,88 @@ class TestSpinChain:
 
         assert_((U_ideal - U_physical).norm() < 1e-12)
 
+    def test_analytical_evo(self):
+        """
+        Test of run_state with exp(-iHt)
+        """
+        N = 3
+        qc = QubitCircuit(N)
+        qc.add_gate("RX", targets=[0], arg_value=np.pi/2)
+        qc.add_gate("CNOT", targets=[0], controls=[1])
+        qc.add_gate("ISWAP", targets=[2, 1])
+        qc.add_gate("CNOT", targets=[0], controls=[2])
+        qc.add_gate("SQRTISWAP", targets=[0, 2])
+        qc.add_gate("RZ", arg_value=np.pi/2, targets=[1])
+
+        # CircularSpinChain
+        test = CircularSpinChain(N)
+        tlist, coeffs = test.load_circuit(qc)
+
+        init_state = rand_ket(2**N)
+        init_state.dims = [[2]*N, [1]*N]
+        rho1 = gate_sequence_product([init_state] + qc.propagators())
+        U_list = test.run_state(
+            init_state=init_state, analytical=True)
+        result = gate_sequence_product(U_list)
+        assert_allclose(
+            fidelity(result, rho1), 1., rtol=1e-6,
+            err_msg="Analytical run_state fails in CircularSpinChain")
+
+        # LinearSpinChain
+        test = LinearSpinChain(N)
+        tlist, coeffs = test.load_circuit(qc)
+
+        init_state = rand_ket(2**N)
+        init_state.dims = [[2]*N, [1]*N]
+        rho1 = gate_sequence_product([init_state] + qc.propagators())
+        U_list = test.run_state(
+            init_state=init_state, analytical=True)
+        result = gate_sequence_product(U_list)
+        assert_allclose(
+            fidelity(result, rho1), 1., rtol=1e-6,
+            err_msg="Analytical run_state fails in LinearSpinChain")
+
+    def test_numerical_evo(self):
+        """
+        Test run_state with qutip solver
+        """
+        N = 3
+        qc = QubitCircuit(N)
+        qc.add_gate("RX", targets=[0], arg_value=np.pi/2)
+        qc.add_gate("CNOT", targets=[0], controls=[1])
+        qc.add_gate("ISWAP", targets=[2, 1])
+        qc.add_gate("CNOT", targets=[0], controls=[2])
+        qc.add_gate("SQRTISWAP", targets=[0, 2])
+        qc.add_gate("RZ", arg_value=np.pi/2, targets=[1])
+
+        # CircularSpinChain
+        test = CircularSpinChain(N)
+        tlist, coeffs = test.load_circuit(qc)
+
+        init_state = rand_ket(2**N)
+        init_state.dims = [[2]*N, [1]*N]
+        rho1 = gate_sequence_product([init_state] + qc.propagators())
+        result = test.run_state(
+            init_state=init_state, analytical=False,
+            options=Options(store_final_state=True)).final_state
+        assert_allclose(
+            fidelity(result, rho1), 1., rtol=1e-6,
+            err_msg="Numerical run_state fails in CircularSpinChain")
+
+        # LinearSpinChain
+        test = LinearSpinChain(N)
+        tlist, coeffs = test.load_circuit(qc)
+
+        init_state = rand_ket(2**N)
+        init_state.dims = [[2]*N, [1]*N]
+        rho1 = gate_sequence_product([init_state] + qc.propagators())
+        result = test.run_state(
+            init_state=init_state, analytical=False,
+            options=Options(store_final_state=True)).final_state
+        assert_allclose(
+            fidelity(result, rho1), 1., rtol=1e-6,
+            err_msg="Numerical run_state fails in LinearSpinChain")
+
 
 if __name__ == "__main__":
     run_module_suite()
