@@ -31,12 +31,14 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
-__all__ = ['wigner', 'qfunc', 'spin_q_function', 'spin_wigner', 'wigner_transform']
+__all__ = ['wigner', 'qfunc', 'spin_q_function',
+           'spin_wigner', 'wigner_transform']
 
 import numpy as np
-from scipy import (zeros, array, arange, exp, real, conj, pi,
-                   copy, sqrt, meshgrid, size, polyval, fliplr, conjugate,
+from numpy import (zeros, array, arange, exp, real, conj, pi,
+                   copy, sqrt, meshgrid, size, conjugate,
                    cos, sin)
+from scipy import polyval, fliplr
 import scipy.sparse as sp
 import scipy.fftpack as ft
 import scipy.linalg as la
@@ -52,6 +54,7 @@ from qutip.operators import jmat
 from scipy.special import factorial
 from qutip.cy.sparse_utils import _csr_get_diag
 import qutip as qt
+from qutip.sparse import eigh
 
 
 def wigner_transform(psi, j, fullparity, steps, slicearray):
@@ -212,9 +215,12 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
 
     g : float
         Scaling factor for `a = 0.5 * g * (x + iy)`, default `g = sqrt(2)`.
+        The value of `g` is related to the value of `hbar` in the commutation
+        relation `[x, y] = i * hbar` via `hbar=2/g^2` giving the default
+        value `hbar=1`.
 
     method : string {'clenshaw', 'iterative', 'laguerre', 'fft'}
-        Select method 'clenshaw' 'iterative', 'laguerre', or 'fft', where 'clenshaw' 
+        Select method 'clenshaw' 'iterative', 'laguerre', or 'fft', where 'clenshaw'
         and 'iterative' use an iterative method to evaluate the Wigner functions for density
         matrices :math:`|m><n|`, while 'laguerre' uses the Laguerre polynomials
         in scipy for the same task. The 'fft' method evaluates the Fourier
@@ -229,7 +235,7 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
         Tells the default solver whether or not to keep the input density
         matrix in sparse format.  As the dimensions of the density matrix
         grow, setthing this flag can result in increased performance.
-    
+
     parfor : bool {False, True}
         Flag for calculating the Laguerre polynomial based Wigner function
         method='laguerre' in parallel using the parfor function.
@@ -275,7 +281,7 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
 
     elif method == 'laguerre':
         return _wigner_laguerre(rho, xvec, yvec, g, parfor)
-        
+
     elif method == 'clenshaw':
         return _wigner_clenshaw(rho, xvec, yvec, g, sparse=sparse)
 
@@ -411,7 +417,7 @@ def _wigner_fourier(psi, xvec, g=np.sqrt(2)):
     if psi.type == 'ket':
         return _psi_wigner_fft(psi.full(), xvec, g)
     elif psi.type == 'oper':
-        eig_vals, eig_vecs = la.eigh(psi.full())
+        eig_vals, eig_vecs = eigh(psi.full())
         W = 0
         for ii in range(psi.shape[0]):
             W1, yvec = _psi_wigner_fft(
@@ -476,19 +482,19 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
     """
     Using Clenshaw summation - numerically stable and efficient
     iterative algorithm to evaluate polynomial series.
-    
+
     The Wigner function is calculated as
-    :math:`W = e^(-0.5*x^2)/pi * \sum_{L} c_L (2x)^L / sqrt(L!)` where 
+    :math:`W = e^(-0.5*x^2)/pi * \sum_{L} c_L (2x)^L / sqrt(L!)` where
     :math:`c_L = \sum_n \\rho_{n,L+n} LL_n^L` where
     :math:`LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]`
-    
+
     """
 
     M = np.prod(rho.shape[0])
     X,Y = np.meshgrid(xvec, yvec)
     #A = 0.5 * g * (X + 1.0j * Y)
     A2 = g * (X + 1.0j * Y) #this is A2 = 2*A
-    
+
     B = np.abs(A2)
     B *= B
     w0 = (2*rho.data[0,-1])*np.ones_like(A2)
@@ -510,17 +516,17 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
                 diag *= 2
             #here c_L = _wig_laguerre_val(L, B, np.diag(rho, L))
             w0 = _wig_laguerre_val(L, B, diag) + w0 * A2 * (L+1)**-0.5
-        
+
     return w0.real * np.exp(-B*0.5) * (g*g*0.5 / pi)
 
 
 def _wig_laguerre_val(L, x, c):
     """
-    this is evaluation of polynomial series inspired by hermval from numpy.    
+    this is evaluation of polynomial series inspired by hermval from numpy.
     Returns polynomial series
     \sum_n b_n LL_n^L,
     where
-    LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]    
+    LL_n^L = (-1)^n sqrt(L!n!/(L+n)!) LaguerreL[n,L,x]
     The evaluation uses Clenshaw recursion
     """
 
@@ -538,10 +544,10 @@ def _wig_laguerre_val(L, x, c):
             k -= 1
             y0,    y1 = c[-i] - y1 * (float((k - 1)*(L + k - 1))/((L+k)*k))**0.5, \
             y0 - y1 * ((L + 2*k -1) - x) * ((L+k)*k)**-0.5
-            
+
     return y0 - y1 * ((L + 1) - x) * (L + 1)**-0.5
-    
-    
+
+
 
 # -----------------------------------------------------------------------------
 # Q FUNCTION
@@ -556,13 +562,17 @@ def qfunc(state, xvec, yvec, g=sqrt(2)):
         A state vector or density matrix.
 
     xvec : array_like
-        x-coordinates at which to calculate the Wigner function.
+        x-coordinates at which to calculate the Husimi-Q function.
 
     yvec : array_like
-        y-coordinates at which to calculate the Wigner function.
+        y-coordinates at which to calculate the Husimi-Q function.
 
     g : float
         Scaling factor for `a = 0.5 * g * (x + iy)`, default `g = sqrt(2)`.
+        The value of `g` is related to the value of `hbar` in the commutation
+        relation `[x, y] = 1j * hbar` via `hbar=2/g^2` giving the default
+        value `hbar=1`.
+
 
     Returns
     --------
@@ -628,14 +638,14 @@ def spin_q_function(rho, theta, phi):
     state : qobj
         A state vector or density matrix for a spin-j quantum system.
     theta : array_like
-        theta-coordinates at which to calculate the Q function.
+        Polar angle at which to calculate the Husimi-Q function.
     phi : array_like
-        phi-coordinates at which to calculate the Q function.
+        Azimuthal angle at which to calculate the Husimi-Q function.
 
     Returns
     -------
     Q, THETA, PHI : 2d-array
-        Values representing the spin Q function at the values specified
+        Values representing the spin Husimi Q function at the values specified
         by THETA and PHI.
 
     """
@@ -647,32 +657,31 @@ def spin_q_function(rho, theta, phi):
         rho = ket2dm(rho)
 
     J = rho.shape[0]
-    j = int((J - 1) / 2)
+    j = (J - 1) / 2
 
     THETA, PHI = meshgrid(theta, phi)
 
     Q = np.zeros_like(THETA, dtype=complex)
 
-    for m1 in range(-j, j+1):
+    for m1 in arange(-j, j+1):
 
         Q += binom(2*j, j+m1) * cos(THETA/2) ** (2*(j-m1)) * sin(THETA/2) ** (2*(j+m1)) * \
-            rho.data[int(j-m1), int(j-m1)]
+             rho.data[int(j-m1), int(j-m1)]
 
-        for m2 in range(m1+1, j+1):
+        for m2 in arange(m1+1, j+1):
 
             Q += (sqrt(binom(2*j, j+m1)) * sqrt(binom(2*j, j+m2)) *
                   cos(THETA/2) ** (2*j-m1-m2) * sin(THETA/2) ** (2*j+m1+m2)) * \
-                (exp(1j * (m2-m1) * PHI) * rho.data[int(j-m1), int(j-m2)] +
-                 exp(1j * (m1-m2) * PHI) * rho.data[int(j-m2), int(j-m1)])
-
-    return Q.real, THETA, PHI
+                  (exp(1j * (m2-m1) * PHI) * rho.data[int(j-m1), int(j-m2)] +
+                   exp(1j * (m1-m2) * PHI) * rho.data[int(j-m2), int(j-m1)])
 
+    return Q.real/pi, THETA, PHI
 
 def _rho_kq(rho, j, k, q):
     v = 0j
 
-    for m1 in range(-j, j+1):
-        for m2 in range(-j, j+1):
+    for m1 in arange(-j, j+1):
+        for m2 in arange(-j, j+1):
             v += (-1)**(j - m1 - q) * clebsch(j, j, k, m1, -m2,
                                               q) * rho.data[m1 + j, m2 + j]
 
@@ -680,16 +689,17 @@ def _rho_kq(rho, j, k, q):
 
 
 def spin_wigner(rho, theta, phi):
-    """Wigner function for spins on the Bloch sphere.
+    """Wigner function for a spin-j system on the 2-sphere of radius j
+       (for j = 1/2 this is the Bloch sphere).
 
     Parameters
     ----------
     state : qobj
         A state vector or density matrix for a spin-j quantum system.
     theta : array_like
-        theta-coordinates at which to calculate the Q function.
+        Polar angle at which to calculate the W function.
     phi : array_like
-        phi-coordinates at which to calculate the Q function.
+        Azimuthal angle at which to calculate the W function.
 
     Returns
     -------
@@ -710,14 +720,15 @@ def spin_wigner(rho, theta, phi):
         rho = ket2dm(rho)
 
     J = rho.shape[0]
-    j = int((J - 1) / 2)
+    j = (J - 1) / 2
 
     THETA, PHI = meshgrid(theta, phi)
 
     W = np.zeros_like(THETA, dtype=complex)
 
     for k in range(int(2 * j)+1):
-        for q in range(-k, k+1):
+        for q in arange(-k, k+1):
+            # sph_harm takes azimuthal angle then polar angle as arguments
             W += _rho_kq(rho, j, k, q) * sph_harm(q, k, PHI, THETA)
 
     return W, THETA, PHI
